@@ -74,8 +74,9 @@
 
 // Bit representations of flags
 enum {
-    LUT_CLIP_BELOW = 1 << 0,
-    LUT_CLIP_ABOVE = 1 << 1
+    LUT_CLIP_OFF,   // LUT does not clip input values
+    LUT_CLIP_BELOW, // LUT clips input values at lower bound
+    LUT_CLIP_ABOVE  // LUT clips input values at upper bound
 };
 
 template<typename T>
@@ -88,8 +89,7 @@ using LUTd = LUT<double>;
 using LUTuc = LUT<uint8_t>;
 
 template<typename T>
-class LUT :
-    public rtengine::NonCopyable
+class LUT
 {
 protected:
     // list of variables ordered to improve cache speed
@@ -111,7 +111,7 @@ public:
     /// The user have to handle it itself, even if some method can (re)initialize it
     bool dirty;
 
-    LUT(int s, int flags = 0xfffffff)
+    LUT(int s, int flags = LUT_CLIP_BELOW | LUT_CLIP_ABOVE, bool initZero = false)
     {
 #ifndef NDEBUG
 
@@ -137,8 +137,11 @@ public:
         sizeiv =  _mm_set1_epi32( (int)(size - 1) );
         sizev = F2V( size - 1 );
 #endif
+        if (initZero) {
+            clear();
+        }
     }
-    void operator ()(int s, int flags = 0xfffffff)
+    void operator ()(int s, int flags = LUT_CLIP_BELOW | LUT_CLIP_ABOVE, bool initZero = false)
     {
 #ifndef NDEBUG
 
@@ -167,6 +170,10 @@ public:
         sizeiv =  _mm_set1_epi32( (int)(size - 1) );
         sizev = F2V( size - 1 );
 #endif
+        if (initZero) {
+            clear();
+        }
+
     }
 
     LUT()
@@ -190,6 +197,8 @@ public:
         }
     }
 
+    explicit LUT(const LUT&) = delete;
+
     void setClip(int flags)
     {
         clip = flags;
@@ -217,7 +226,7 @@ public:
         return size > 0 ? upperBound : 0;
     }
 
-    LUT<T> & operator=(LUT<T> &rhs)
+    LUT<T> & operator=(const LUT<T>& rhs)
     {
         if (this != &rhs) {
             if (rhs.size > this->size) {
@@ -249,7 +258,7 @@ public:
 
     // handy to sum up per thread histograms. #pragma omp simd speeds up the loop by about factor 3 for LUTu (uint32_t).
     template<typename U = T, typename = typename std::enable_if<std::is_same<U, std::uint32_t>::value>::type>
-    LUT<T> & operator+=(LUT<T> &rhs)
+    LUT<T> & operator+=(const LUT<T>& rhs)
     {
         if (rhs.size == this->size) {
 #ifdef _OPENMP
@@ -311,7 +320,7 @@ public:
 
         // Clamp and convert to integer values. Extract out of SSE register because all
         // lookup operations use regular addresses.
-        vfloat clampedIndexes = vmaxf(ZEROV, vminf(maxsv, indexv));
+        vfloat clampedIndexes = vclampf(indexv, ZEROV, maxsv); // this automagically uses ZEROV in case indexv is NaN
         vint indexes = _mm_cvttps_epi32(clampedIndexes);
         int indexArray[4];
         _mm_storeu_si128(reinterpret_cast<__m128i*>(&indexArray[0]), indexes);
@@ -343,7 +352,7 @@ public:
 
         // Clamp and convert to integer values. Extract out of SSE register because all
         // lookup operations use regular addresses.
-        vfloat clampedIndexes = vmaxf(ZEROV, vminf(maxsv, indexv));
+        vfloat clampedIndexes = vclampf(indexv, ZEROV, maxsv); // this automagically uses ZEROV in case indexv is NaN
         vint indexes = _mm_cvttps_epi32(clampedIndexes);
         int indexArray[4];
         _mm_storeu_si128(reinterpret_cast<__m128i*>(&indexArray[0]), indexes);
@@ -363,7 +372,7 @@ public:
         vfloat lower = _mm_castsi128_ps(_mm_unpacklo_epi64(temp0, temp1));
         vfloat upper = _mm_castsi128_ps(_mm_unpackhi_epi64(temp0, temp1));
 
-        vfloat diff = vmaxf(ZEROV, vminf(sizev, indexv)) - _mm_cvtepi32_ps(indexes);
+        vfloat diff = vclampf(indexv, ZEROV, sizev) - _mm_cvtepi32_ps(indexes); // this automagically uses ZEROV in case indexv is NaN
         return vintpf(diff, upper, lower);
     }
 
@@ -374,7 +383,7 @@ public:
 
         // Clamp and convert to integer values. Extract out of SSE register because all
         // lookup operations use regular addresses.
-        vfloat clampedIndexes = vmaxf(ZEROV, vminf(maxsv, indexv));
+        vfloat clampedIndexes = vclampf(indexv, ZEROV, maxsv); // this automagically uses ZEROV in case indexv is NaN
         vint indexes = _mm_cvttps_epi32(clampedIndexes);
         int indexArray[4];
         _mm_storeu_si128(reinterpret_cast<__m128i*>(&indexArray[0]), indexes);
@@ -397,94 +406,35 @@ public:
         vfloat diff = indexv - _mm_cvtepi32_ps(indexes);
         return vintpf(diff, upper, lower);
     }
+
+    // vectorized LUT access with integer indices. Clips at lower and upper bounds
 #ifdef __SSE4_1__
     template<typename U = T, typename = typename std::enable_if<std::is_same<U, float>::value>::type>
-    vfloat operator[](vint idxv ) const
+    vfloat operator[](vint idxv) const
     {
-        vfloat tempv, p1v;
         idxv = _mm_max_epi32( _mm_setzero_si128(), _mm_min_epi32(idxv, sizeiv));
-        // access the LUT 4 times and shuffle the values into p1v
-
-        int idx;
-
-        // get 4th value
-        idx = _mm_extract_epi32(idxv, 3);
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS(tempv, _MM_SHUFFLE(0, 0, 0, 0));
-        // now p1v is 3 3 3 3
-
-        // get 3rd value
-        idx = _mm_extract_epi32(idxv, 2);
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = _mm_move_ss( p1v, tempv);
-        // now p1v is 3 3 3 2
-
-        // get 2nd value
-        idx = _mm_extract_epi32(idxv, 1);
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS( p1v, _MM_SHUFFLE(1, 0, 1, 0));
-        // now p1v is 3 2 3 2
-        p1v = _mm_move_ss( p1v, tempv );
-        // now p1v is 3 2 3 1
-
-        // get 1st value
-        idx = _mm_cvtsi128_si32(idxv);
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS( p1v, _MM_SHUFFLE(3, 2, 0, 0));
-        // now p1v is 3 2 1 1
-        p1v = _mm_move_ss( p1v, tempv );
-        // now p1v is 3 2 1 0
-
-        return p1v;
+        // access the LUT 4 times. Trust the compiler. It generates good code here, better than hand written SSE code
+        return _mm_setr_ps(data[_mm_extract_epi32(idxv,0)], data[_mm_extract_epi32(idxv,1)], data[_mm_extract_epi32(idxv,2)], data[_mm_extract_epi32(idxv,3)]);
     }
 #else
     template<typename U = T, typename = typename std::enable_if<std::is_same<U, float>::value>::type>
-    vfloat operator[](vint idxv ) const
+    vfloat operator[](vint idxv) const
     {
-        vfloat tempv, p1v;
-        tempv = _mm_cvtepi32_ps(idxv);
-        tempv = _mm_min_ps( tempv, sizev );
-        idxv = _mm_cvttps_epi32(_mm_max_ps( tempv, _mm_setzero_ps( )  ));
-        // access the LUT 4 times and shuffle the values into p1v
-
-        int idx;
-
-        // get 4th value
-        idx = _mm_cvtsi128_si32 (_mm_shuffle_epi32(idxv, _MM_SHUFFLE(3, 3, 3, 3)));
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS(tempv, _MM_SHUFFLE(0, 0, 0, 0));
-        // now p1v is 3 3 3 3
-
-        // get 3rd value
-        idx = _mm_cvtsi128_si32 (_mm_shuffle_epi32(idxv, _MM_SHUFFLE(2, 2, 2, 2)));
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = _mm_move_ss( p1v, tempv);
-        // now p1v is 3 3 3 2
-
-        // get 2nd value
-        idx = _mm_cvtsi128_si32 (_mm_shuffle_epi32(idxv, _MM_SHUFFLE(1, 1, 1, 1)));
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS( p1v, _MM_SHUFFLE(1, 0, 1, 0));
-        // now p1v is 3 2 3 2
-        p1v = _mm_move_ss( p1v, tempv );
-        // now p1v is 3 2 3 1
-
-        // get 1st value
-        idx = _mm_cvtsi128_si32 (idxv);
-        tempv = _mm_load_ss(&data[idx]);
-        p1v = PERMUTEPS( p1v, _MM_SHUFFLE(3, 2, 0, 0));
-        // now p1v is 3 2 1 1
-        p1v = _mm_move_ss( p1v, tempv );
-        // now p1v is 3 2 1 0
-
-        return p1v;
+        // convert to float because SSE2 has no min/max for 32bit integers
+        vfloat tempv = vclampf(_mm_cvtepi32_ps(idxv), ZEROV, sizev); // this automagically uses ZEROV in case idxv is NaN (which will never happen because it is a vector of int)
+        idxv = _mm_cvttps_epi32(tempv);
+        // access the LUT 4 times. Trust the compiler. It generates good code here, better than hand written SSE code
+        return _mm_setr_ps(data[_mm_cvtsi128_si32(idxv)],
+                           data[_mm_cvtsi128_si32(_mm_shuffle_epi32(idxv, _MM_SHUFFLE(1, 1, 1, 1)))],
+                           data[_mm_cvtsi128_si32(_mm_shuffle_epi32(idxv, _MM_SHUFFLE(2, 2, 2, 2)))],
+                           data[_mm_cvtsi128_si32(_mm_shuffle_epi32(idxv, _MM_SHUFFLE(3, 3, 3, 3)))]);
     }
 #endif
 #endif
 
     // use with float indices
-    template<typename U = T, typename = typename std::enable_if<std::is_same<U, float>::value>::type>
-    T operator[](float index) const
+    template<typename U = T, typename V, typename = typename std::enable_if<std::is_floating_point<V>::value && std::is_same<U, float>::value>::type>
+    T operator[](V index) const
     {
         int idx = (int)index;  // don't use floor! The difference in negative space is no problems here
 
@@ -676,7 +626,7 @@ public:
     }
 
     // share the buffer with another LUT, handy for same data but different clip flags
-    void share(const LUT<T> &source, int flags = 0xfffffff)
+    void share(const LUT<T> &source, int flags = LUT_CLIP_BELOW | LUT_CLIP_ABOVE)
     {
         if (owner && data) {
             delete[] data;
