@@ -38,7 +38,7 @@
 namespace rtengine
 {
 
-void RawImageSource::amaze_demosaic_RT(int winx, int winy, int winw, int winh, array2D<float> &rawData, array2D<float> &red, array2D<float> &green, array2D<float> &blue)
+void RawImageSource::amaze_demosaic_RT(int winx, int winy, int winw, int winh, const array2D<float> &rawData, array2D<float> &red, array2D<float> &green, array2D<float> &blue)
 {
     BENCHFUN
 
@@ -1584,6 +1584,9 @@ void RawImageSource::amaze_demosaic_RT(int winx, int winy, int winw, int winh, a
         // clean up
         free(buffer);
     }
+    if(border < 4) {
+        border_interpolate2(W, H, 3, rawData, red, green, blue);
+    }
 
     if(plistener) {
         plistener->setProgress(1.0);
