@@ -20,11 +20,13 @@
 #pragma once
 
 #include <array>
+#include <glibmm.h>
 
 #include "rt_math.h"
 #include "LUT.h"
 #include "labimage.h"
 #include "iccmatrices.h"
+#include "lcms2.h"
 #include "sleef.c"
 
 #define SAT(a,b,c) ((float)max(a,b,c)-(float)min(a,b,c))/(float)max(a,b,c)
@@ -97,6 +99,10 @@ private:
 #ifdef __SSE2__
     static vfloat hue2rgb(vfloat p, vfloat q, vfloat t);
 #endif
+
+    static float computeXYZ2Lab(float f);
+    static float computeXYZ2LabY(float f);
+    
 public:
 
     typedef enum Channel {
@@ -199,6 +205,10 @@ public:
         return r * 0.2126729 + g * 0.7151521 + b * 0.0721750;
     }
 
+    static float rgbLuminance(float r, float g, float b, const double workingspace[3][3])
+    {
+        return r * workingspace[1][0] + g * workingspace[1][1] + b * workingspace[1][2];
+    }
 
     /**
     * @brief Convert red/green/blue to L*a*b
@@ -607,6 +617,7 @@ public:
     */
     static void XYZ2Lab(float x, float y, float z, float &L, float &a, float &b);
     static void RGB2Lab(float *X, float *Y, float *Z, float *L, float *a, float *b, const float wp[3][3], int width);
+    static void RGB2L(float *X, float *Y, float *Z, float *L, const float wp[3][3], int width);
 
     /**
     * @brief Convert Lab in Yuv
@@ -640,7 +651,9 @@ public:
     * @param h 'h' channel return value, in [-PI ; +PI] (return value)
     */
     static void Lab2Lch(float a, float b, float &c, float &h);
-
+#ifdef __SSE2__
+    static void Lab2Lch(float *a, float *b, float *c, float *h, int w);
+#endif
 
     /**
     * @brief Convert 'c' and 'h' channels of the Lch color space to the 'a' and 'b' channels of the L*a*b color space (channel 'L' is identical [0 ; 32768])
@@ -1069,25 +1082,35 @@ public:
 
     /**
     * @brief sRGB gamma
-    * See also calcGamma above with the following values: pwr=2.4  ts=12.92  mode=0.003041  imax=0.055011
+    * See also calcGamma above with the following values: pwr=2.399  ts=12.92310  mode=0.003041  imax=0.055
     * @param x red, green or blue channel's value [0 ; 1]
     * @return the gamma modified's value [0 ; 1]
     */
     static inline double gamma2     (double x)      //  g3                  1+g4
     {
-        return x <= 0.003041 ? x * 12.92 : 1.055011 * exp(log(x) / sRGBGammaCurve) - 0.055011;
+      //  return x <= 0.003041 ? x * 12.92310 : 1.055 * exp(log(x) / 2.39990) - 0.055;//calculate with calcgamma
+        //return x <= 0.0031308 ? x * 12.92310 : 1.055 * exp(log(x) / sRGBGammaCurve) - 0.055;//standard discontinuous
+		//very small differences between the 2
+        return x <= 0.003040 ? x * 12.92310 : 1.055 * exp(log(x) / sRGBGammaCurve) - 0.055;//continuous
+      //  return x <= 0.003041 ? x * 12.92310 : 1.055011 * exp(log(x) / sRGBGammaCurve) - 0.055011;//continuous
+		
     }
 
 
     /**
     * @brief Inverse sRGB gamma
-    * See also calcGamma above with the following values: pwr=2.4  ts=12.92  mode=0.003041  imax=0.055011
+    * See also calcGamma above with the following values: pwr=2.3999  ts=12.92310  mode=0.003041  imax=0.055
     * @param x red, green or blue channel's value [0 ; 1]
     * @return the inverse gamma modified's value [0 ; 1]
     */
     static inline double igamma2    (double x)      //g2
     {
-        return x <= 0.039293 ? x / 12.92 : exp(log((x + 0.055011) / 1.055011) * sRGBGammaCurve);
+       // return x <= 0.039289 ? x / 12.92310 : exp(log((x + 0.055) / 1.055) * 2.39990);//calculate with calcgamma
+       // return x <= 0.04045 ? x / 12.92310 : exp(log((x + 0.055) / 1.055) * sRGBGammaCurve);//standard discontinuous
+		//very small differences between the 4
+        return x <= 0.039286 ? x / 12.92310 : exp(log((x + 0.055) / 1.055) * sRGBGammaCurve);//continuous
+      //  return x <= 0.039293 ? x / 12.92310 : exp(log((x + 0.055011) / 1.055011) * sRGBGammaCurve);//continuous
+		
     }
 
 
@@ -1380,6 +1403,7 @@ public:
 #else
     static void AllMunsellLch (bool lumaMuns, float Lprov1, float Loldd, float HH, float Chprov1, float CC, float &correctionHueChroma, float &correctlum);
 #endif
+    static void AllMunsellLch (float Lprov1, float HH, float Chprov1, float CC, float &correctionHueChroma);
 
 
     /**
@@ -1412,6 +1436,7 @@ public:
     static void gamutLchonly  (float HH, float2 sincosval, float &Lprov1, float &Chprov1, float &R, float &G, float &B, const double wip[3][3], const bool isHLEnabled, const float lowerCoef, const float higherCoef);
     static void gamutLchonly  (float2 sincosval, float &Lprov1, float &Chprov1, const float wip[3][3], const bool isHLEnabled, const float lowerCoef, const float higherCoef);
 #endif
+    static void gamutLchonly  (float HH, float2 sincosval, float &Lprov1, float &Chprov1, float &saturation, const float wip[3][3], const bool isHLEnabled, const float lowerCoef, const float higherCoef);
 
 
     /**
