@@ -32,6 +32,19 @@ constexpr double d50_d65[3][3] =  {
     { -0.0092345,  0.0150436,  0.7521316}
 };
 
+//J.Desmis 04 2018
+constexpr double d50_d60[3][3] =  {
+    { 1.034368,  0.016908, -0.037658},
+    {0.021752,  0.992183, -0.012785},
+    { -0.006971,  0.011377,  0.812150}
+};
+
+constexpr double d60_d50[3][3] =  {
+    { 0.96743198, -0.01699717, 0.044590689},
+    {-0.02109893, 1.008067172, 0.014890785},
+    {0.008598998, -0.01426777, 1.231474467}
+};
+
 // Color space conversion to/from XYZ; color spaces adapted to D65
 constexpr double xyz_sRGBd65[3][3] = {
     {0.4124564,  0.3575761,  0.1804375},
@@ -66,6 +79,31 @@ constexpr double xyz_sRGB[3][3] = {
     {0.0139322,  0.0971045,  0.7141733}
 };
 
+constexpr double xyz_ACESp0[3][3] = {
+    {0.9908526, 0.0122334, -0.0388654},
+    {0.3618807, 0.72255045  , -0.0843859},
+    {-0.0027093, 0.0082323, 0.8196880}
+};
+
+constexpr double ACESp0_xyz[3][3] = {
+    {1.01583320, -0.01772807, 0.04634052},
+    {-0.50781231, 1.39131494  , 0.11915641},
+    {0.00845768,  -0.01403193, 1.21893277}
+};
+
+constexpr double xyz_ACESp1[3][3] = {
+    {0.689697, 0.149944, 0.124559},
+    {0.284448, 0.671758  , 0.043794},
+    {-0.006043, 0.009998, 0.820945}
+};
+
+constexpr double ACESp1_xyz[3][3] = {
+    {1.5932624, -0.352316, -0.222945},
+    {-0.6759496, 1.639286  , 0.01511026},
+    {0.0199602,  -0.0225577, 1.2162833}
+};
+
+
 constexpr double sRGB_xyz[3][3] = {
     {3.1338561, -1.6168667, -0.4906146},
     { -0.9787684,  1.9161415,  0.0334540},
