@@ -82,22 +82,18 @@ public:
     // Read the raw dump of the data
     void readData  (FILE *fh) {}
     // Write a raw dump of the data
-    void writeData (FILE *fh) {}
+    void writeData (FILE *fh) const {}
 
     virtual void normalizeInt (int srcMinVal, int srcMaxVal) {};
     virtual void normalizeFloat (float srcMinVal, float srcMaxVal) {};
-    virtual void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, int compression) {}
+    virtual void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, int compression) const {}
     virtual void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
                                 std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue,
-                                int tran) {}
-    virtual void getAutoWBMultipliers (double &rm, double &gm, double &bm)
+                                int tran) const {}
+    virtual void getAutoWBMultipliers (double &rm, double &gm, double &bm) const
     {
         rm = gm = bm = 1.0;
     }
-    virtual const char* getType () const
-    {
-        return "unknown";
-    }
 
 };
 
@@ -119,7 +115,7 @@ inline void ImageDatas::convertTo(unsigned char src, unsigned short& dst) const
 template<>
 inline void ImageDatas::convertTo(float src, unsigned char& dst) const
 {
-    dst = uint16ToUint8Rounded(src);
+    dst = uint16ToUint8Rounded(CLIP(src));
 }
 template<>
 inline void ImageDatas::convertTo(unsigned char src, float& dst) const
@@ -232,7 +228,7 @@ public:
     }
 
     // Send back the row stride. WARNING: unit = byte, not element!
-    int getRowStride ()
+    int getRowStride () const
     {
         return rowstride;
     }
@@ -261,7 +257,7 @@ public:
 
     /* If any of the required allocation fails, "width" and "height" are set to -1, and all remaining buffer are freed
      * Can be safely used to reallocate an existing image */
-    void allocate (int W, int H)
+    void allocate (int W, int H) override
     {
 
         if (W == width && H == height) {
@@ -316,7 +312,7 @@ public:
     }
 
     /** Copy the data to another PlanarWhateverData */
-    void copyData(PlanarWhateverData<T> *dest)
+    void copyData(PlanarWhateverData<T> *dest) const
     {
         assert (dest != NULL);
         // Make sure that the size is the same, reallocate if necessary
@@ -331,7 +327,7 @@ public:
         }
     }
 
-    void rotate (int deg)
+    void rotate (int deg) override
     {
 
         if (deg == 90) {
@@ -389,7 +385,7 @@ public:
     }
 
     template <class IC>
-    void resizeImgTo (int nw, int nh, TypeInterpolation interp, PlanarWhateverData<IC> *imgPtr)
+    void resizeImgTo (int nw, int nh, TypeInterpolation interp, PlanarWhateverData<IC> *imgPtr) const
     {
         //printf("resizeImgTo: resizing %s image data (%d x %d) to %s (%d x %d)\n", getType(), width, height, imgPtr->getType(), imgPtr->width, imgPtr->height);
         if (width == nw && height == nh) {
@@ -450,7 +446,7 @@ public:
         }
     }
 
-    void hflip ()
+    void hflip () override
     {
         int width2 = width / 2;
 
@@ -474,7 +470,7 @@ public:
 #endif
     }
 
-    void vflip ()
+    void vflip () override
     {
 
         int height2 = height / 2;
@@ -499,17 +495,7 @@ public:
 #endif
     }
 
-    void calcHist(unsigned int *hist16)
-    {
-        for (int row = 0; row < height; row++)
-            for (int col = 0; col < width; col++) {
-                unsigned short idx;
-                convertTo(v(row, col), idx);
-                hist16[idx]++;
-            }
-    }
-
-    void transformPixel (int x, int y, int tran, int& tx, int& ty)
+    void transformPixel (int x, int y, int tran, int& tx, int& ty) const
     {
 
         if (!tran) {
@@ -552,7 +538,7 @@ public:
         }
     }
 
-    void getPipetteData (T &value, int posX, int posY, int squareSize, int tran)
+    void getPipetteData (T &value, int posX, int posY, int squareSize, int tran) const
     {
         int x;
         int y;
@@ -573,20 +559,28 @@ public:
         value = n ? T(accumulator / float(n)) : T(0);
     }
 
-    void readData   (FILE *f)
+    void readData (FILE *f)
     {
         for (int i = 0; i < height; i++) {
             fread (v(i), sizeof(T), width, f);
         }
     }
 
-    void writeData  (FILE *f)
+    void writeData (FILE *f) const
     {
         for (int i = 0; i < height; i++) {
             fwrite (v(i), sizeof(T), width, f);
         }
     }
 
+    void fill (T value) {
+        for (int i = 0; i < height; i++) {
+            for (int j = 0; j < width; j++) {
+                v(i, j) = value;
+            }
+        }
+    }
+
 };
 
 
@@ -614,12 +608,12 @@ public:
     }
 
     // Send back the row stride. WARNING: unit = byte, not element!
-    int getRowStride ()
+    int getRowStride () const
     {
         return rowstride;
     }
     // Send back the plane stride. WARNING: unit = byte, not element!
-    int getPlaneStride ()
+    int getPlaneStride () const
     {
         return planestride;
     }
@@ -654,7 +648,7 @@ public:
 
     /* If any of the required allocation fails, "width" and "height" are set to -1, and all remaining buffer are freed
      * Can be safely used to reallocate an existing image */
-    void allocate (int W, int H)
+    void allocate (int W, int H) override
     {
 
         if (W == width && H == height) {
@@ -724,7 +718,7 @@ public:
     }
 
     /** Copy the data to another PlanarRGBData */
-    void copyData(PlanarRGBData<T> *dest)
+    void copyData(PlanarRGBData<T> *dest) const
     {
         assert (dest != nullptr);
         // Make sure that the size is the same, reallocate if necessary
@@ -742,7 +736,7 @@ public:
         }
     }
 
-    void rotate (int deg)
+    void rotate (int deg) override
     {
 
         if (deg == 90) {
@@ -812,7 +806,7 @@ public:
     }
 
     template <class IC>
-    void resizeImgTo (int nw, int nh, TypeInterpolation interp, IC *imgPtr)
+    void resizeImgTo (int nw, int nh, TypeInterpolation interp, IC *imgPtr) const
     {
         //printf("resizeImgTo: resizing %s image data (%d x %d) to %s (%d x %d)\n", getType(), width, height, imgPtr->getType(), imgPtr->width, imgPtr->height);
         if (width == nw && height == nh) {
@@ -861,15 +855,15 @@ public:
             // This case should never occur!
             for (int i = 0; i < nh; i++) {
                 for (int j = 0; j < nw; j++) {
-                    r(i, j) = 0;
-                    g(i, j) = 0;
-                    b(i, j) = 0;
+                    imgPtr->r(i, j) = 0;
+                    imgPtr->g(i, j) = 0;
+                    imgPtr->b(i, j) = 0;
                 }
             }
         }
     }
 
-    void hflip ()
+    void hflip () override
     {
         int width2 = width / 2;
 
@@ -901,7 +895,7 @@ public:
 #endif
     }
 
-    void vflip ()
+    void vflip () override
     {
 
         int height2 = height / 2;
@@ -934,7 +928,7 @@ public:
 #endif
     }
 
-    void calcGrayscaleHist(unsigned int *hist16)
+    void calcGrayscaleHist(unsigned int *hist16) const
     {
         for (int row = 0; row < height; row++)
             for (int col = 0; col < width; col++) {
@@ -948,7 +942,7 @@ public:
             }
     }
 
-    void computeAutoHistogram (LUTu & histogram, int& histcompr)
+    void computeAutoHistogram (LUTu & histogram, int& histcompr) const
     {
         histcompr = 3;
 
@@ -967,7 +961,7 @@ public:
             }
     }
 
-    void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, const int compression)
+    void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, const int compression) const override
     {
         histogram.clear();
         avg_r = avg_g = avg_b = 0.;
@@ -999,7 +993,7 @@ public:
             }
     }
 
-    void getAutoWBMultipliers (double &rm, double &gm, double &bm)
+    void getAutoWBMultipliers (double &rm, double &gm, double &bm) const override
     {
 
         double avg_r = 0.;
@@ -1030,7 +1024,7 @@ public:
         bm = avg_b / double(n);
     }
 
-    void transformPixel (int x, int y, int tran, int& tx, int& ty)
+    void transformPixel (int x, int y, int tran, int& tx, int& ty) const
     {
 
         if (!tran) {
@@ -1073,9 +1067,9 @@ public:
         }
     }
 
-    virtual void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
+    void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
                                 std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue,
-                                int tran)
+                                int tran) const override
     {
         int x;
         int y;
@@ -1112,7 +1106,7 @@ public:
         }
     }
 
-    void getPipetteData (T &valueR, T &valueG, T &valueB, int posX, int posY, int squareSize, int tran)
+    void getPipetteData (T &valueR, T &valueG, T &valueB, int posX, int posY, int squareSize, int tran) const
     {
         int x;
         int y;
@@ -1139,22 +1133,28 @@ public:
         valueB = n ? T(accumulatorB / float(n)) : T(0);
     }
 
-    void readData   (FILE *f)
+    void readData (FILE *f)
     {
         for (int i = 0; i < height; i++) {
-            fread (r(i), sizeof(T), width, f);
+            if (fread(r(i), sizeof(T), width, f) < static_cast<size_t>(width)) {
+                break;
+            }
         }
 
         for (int i = 0; i < height; i++) {
-            fread (g(i), sizeof(T), width, f);
+            if (fread(g(i), sizeof(T), width, f) < static_cast<size_t>(width)) {
+                break;
+            }
         }
 
         for (int i = 0; i < height; i++) {
-            fread (b(i), sizeof(T), width, f);
+            if (fread(b(i), sizeof(T), width, f) < static_cast<size_t>(width)) {
+                break;
+            }
         }
     }
 
-    void writeData  (FILE *f)
+    void writeData (FILE *f) const
     {
         for (int i = 0; i < height; i++) {
             fwrite (r(i), sizeof(T), width, f);
@@ -1297,7 +1297,7 @@ public:
      * If any of the required allocation fails, "width" and "height" are set to -1, and all remaining buffer are freed
      * Can be safely used to reallocate an existing image or to free up it's memory with "allocate (0,0);"
      */
-    void allocate (int W, int H)
+    void allocate (int W, int H) override
     {
 
         if (W == width && H == height) {
@@ -1337,7 +1337,7 @@ public:
     }
 
     /** Copy the data to another ChunkyRGBData */
-    void copyData(ChunkyRGBData<T> *dest)
+    void copyData(ChunkyRGBData<T> *dest) const
     {
         assert (dest != nullptr);
         // Make sure that the size is the same, reallocate if necessary
@@ -1351,7 +1351,7 @@ public:
         memcpy (dest->data, data, 3 * width * height * sizeof(T));
     }
 
-    void rotate (int deg)
+    void rotate (int deg) override
     {
 
         if (deg == 90) {
@@ -1413,7 +1413,7 @@ public:
     }
 
     template <class IC>
-    void resizeImgTo (int nw, int nh, TypeInterpolation interp, IC *imgPtr)
+    void resizeImgTo (int nw, int nh, TypeInterpolation interp, IC *imgPtr) const
     {
         //printf("resizeImgTo: resizing %s image data (%d x %d) to %s (%d x %d)\n", getType(), width, height, imgPtr->getType(), imgPtr->width, imgPtr->height);
         if (width == nw && height == nh) {
@@ -1477,15 +1477,15 @@ public:
             // This case should never occur!
             for (int i = 0; i < nh; i++) {
                 for (int j = 0; j < nw; j++) {
-                    r(i, j) = 0;
-                    g(i, j) = 0;
-                    b(i, j) = 0;
+                    imgPtr->r(i, j) = 0;
+                    imgPtr->g(i, j) = 0;
+                    imgPtr->b(i, j) = 0;
                 }
             }
         }
     }
 
-    void hflip ()
+    void hflip () override
     {
         int width2 = width / 2;
 
@@ -1521,7 +1521,7 @@ public:
         }
     }
 
-    void vflip ()
+    void vflip () override
     {
 
         AlignedBuffer<T> lBuffer(3 * width);
@@ -1537,7 +1537,7 @@ public:
         }
     }
 
-    void calcGrayscaleHist(unsigned int *hist16)
+    void calcGrayscaleHist(unsigned int *hist16) const
     {
         for (int row = 0; row < height; row++)
             for (int col = 0; col < width; col++) {
@@ -1551,7 +1551,7 @@ public:
             }
     }
 
-    void computeAutoHistogram (LUTu & histogram, int& histcompr)
+    void computeAutoHistogram (LUTu & histogram, int& histcompr) const
     {
         histcompr = 3;
 
@@ -1570,7 +1570,7 @@ public:
             }
     }
 
-    void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, const int compression)
+    void computeHistogramAutoWB (double &avg_r, double &avg_g, double &avg_b, int &n, LUTu &histogram, const int compression) const override
     {
         histogram.clear();
         avg_r = avg_g = avg_b = 0.;
@@ -1602,7 +1602,7 @@ public:
             }
     }
 
-    void getAutoWBMultipliers (double &rm, double &gm, double &bm)
+    void getAutoWBMultipliers (double &rm, double &gm, double &bm) const override
     {
 
         double avg_r = 0.;
@@ -1633,7 +1633,7 @@ public:
         bm = avg_b / double(n);
     }
 
-    void transformPixel (int x, int y, int tran, int& tx, int& ty)
+    void transformPixel (int x, int y, int tran, int& tx, int& ty) const
     {
 
         if (!tran) {
@@ -1676,9 +1676,9 @@ public:
         }
     }
 
-    virtual void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
+    void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
                                 std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue,
-                                int tran)
+                                int tran) const override
     {
         int x;
         int y;
@@ -1715,14 +1715,16 @@ public:
         }
     }
 
-    void readData   (FILE *f)
+    void readData (FILE *f)
     {
         for (int i = 0; i < height; i++) {
-            fread (r(i), sizeof(T), 3 * width, f);
+            if (fread(r(i), sizeof(T), 3 * width, f) < 3 * static_cast<size_t>(width)) {
+                break;
+            }
         }
     }
 
-    void writeData  (FILE *f)
+    void writeData (FILE *f) const
     {
         for (int i = 0; i < height; i++) {
             fwrite (r(i), sizeof(T), 3 * width, f);
@@ -1743,30 +1745,31 @@ public:
     /** @brief Returns a mutex that can is useful in many situations. No image operations shuold be performed without locking this mutex.
       * @return The mutex */
     virtual MyMutex& getMutex () = 0;
-    virtual cmsHPROFILE getProfile () = 0;
+    virtual cmsHPROFILE getProfile () const = 0;
     /** @brief Returns the bits per pixel of the image.
       * @return The bits per pixel of the image */
-    virtual int getBitsPerPixel () = 0;
+    virtual int getBitsPerPixel () const = 0;
     /** @brief Saves the image to file. It autodetects the format (jpg, tif, png are supported).
       * @param fname is the name of the file
         @return the error code, 0 if none */
-    virtual int saveToFile (Glib::ustring fname) = 0;
+    virtual int saveToFile (const Glib::ustring &fname) const = 0;
     /** @brief Saves the image to file in a png format.
       * @param fname is the name of the file
       * @param compression is the amount of compression (0-6), -1 corresponds to the default
       * @param bps can be 8 or 16 depending on the bits per pixels the output file will have
         @return the error code, 0 if none */
-    virtual int saveAsPNG  (Glib::ustring fname, int bps = -1) = 0;
+    virtual int saveAsPNG (const Glib::ustring &fname, int bps = -1) const = 0;
     /** @brief Saves the image to file in a jpg format.
       * @param fname is the name of the file
       * @param quality is the quality of the jpeg (0...100), set it to -1 to use default
         @return the error code, 0 if none */
-    virtual int saveAsJPEG (Glib::ustring fname, int quality = 100, int subSamp = 3 ) = 0;
+    virtual int saveAsJPEG (const Glib::ustring &fname, int quality = 100, int subSamp = 3 ) const = 0;
     /** @brief Saves the image to file in a tif format.
       * @param fname is the name of the file
       * @param bps can be 8 or 16 depending on the bits per pixels the output file will have
+      * @param isFloat is true for saving float images. Will be ignored by file format not supporting float data
         @return the error code, 0 if none */
-    virtual int saveAsTIFF (Glib::ustring fname, int bps = -1, bool uncompressed = false) = 0;
+    virtual int saveAsTIFF (const Glib::ustring &fname, int bps = -1, bool isFloat = false, bool uncompressed = false) const = 0;
     /** @brief Sets the progress listener if you want to follow the progress of the image saving operations (optional).
       * @param pl is the pointer to the class implementing the ProgressListener interface */
     virtual void setSaveProgressListener (ProgressListener* pl) = 0;
@@ -1779,14 +1782,14 @@ public:
 class IImagefloat : public IImage, public PlanarRGBData<float>
 {
 public:
-    virtual ~IImagefloat() {}
+    ~IImagefloat() override {}
 };
 
 /** @brief This class represents an image having a classical 8 bits/pixel representation */
 class IImage8 : public IImage, public ChunkyRGBData<unsigned char>
 {
 public:
-    virtual ~IImage8() {}
+    ~IImage8() override {}
 };
 
 /** @brief This class represents an image having a 16 bits/pixel planar representation.
@@ -1794,7 +1797,7 @@ public:
 class IImage16 : public IImage, public PlanarRGBData<unsigned short>
 {
 public:
-    virtual ~IImage16() {}
+    ~IImage16() override {}
 };
 
 }
