@@ -201,7 +201,7 @@ void png_read_data(png_struct_def  *png_ptr, unsigned char *data, size_t length)
 void png_write_data(png_struct_def *png_ptr, unsigned char *data, size_t length);
 void png_flush(png_struct_def *png_ptr);
 
-int ImageIO::getPNGSampleFormat (Glib::ustring fname, IIOSampleFormat &sFormat, IIOSampleArrangement &sArrangement)
+int ImageIO::getPNGSampleFormat (const Glib::ustring &fname, IIOSampleFormat &sFormat, IIOSampleArrangement &sArrangement)
 {
     FILE *file = g_fopen (fname.c_str (), "rb");
 
@@ -273,7 +273,7 @@ int ImageIO::getPNGSampleFormat (Glib::ustring fname, IIOSampleFormat &sFormat,
     }
 }
 
-int ImageIO::loadPNG  (Glib::ustring fname)
+int ImageIO::loadPNG  (const Glib::ustring &fname)
 {
 
     FILE *file = g_fopen (fname.c_str (), "rb");
@@ -447,7 +447,7 @@ void my_error_exit (j_common_ptr cinfo)
     (*cinfo->err->output_message) (cinfo);
 
     /* Return control to the setjmp point */
-#if defined( WIN32 ) && defined( __x86_64__ )
+#if defined( WIN32 ) && defined( __x86_64__ ) && !defined(__clang__)
     __builtin_longjmp(myerr->setjmp_buffer, 1);
 #else
     longjmp(myerr->setjmp_buffer, 1);
@@ -471,7 +471,7 @@ int ImageIO::loadJPEGFromMemory (const char* buffer, int bufsize)
     jerr.pub.error_exit = my_error_exit;
 
     /* Establish the setjmp return context for my_error_exit to use. */
-#if defined( WIN32 ) && defined( __x86_64__ )
+#if defined( WIN32 ) && defined( __x86_64__ ) && !defined(__clang__)
 
     if (__builtin_setjmp(jerr.setjmp_buffer)) {
 #else
@@ -523,7 +523,7 @@ int ImageIO::loadJPEGFromMemory (const char* buffer, int bufsize)
             return IMIO_READERROR;
         }
 
-        setScanline (cinfo.output_scanline - 1, row, 8);
+        setScanline (cinfo.output_scanline - 1, row, 8, cinfo.num_components);
 
         if (pl && !(cinfo.output_scanline % 100)) {
             pl->setProgress ((double)(cinfo.output_scanline) / cinfo.output_height);
@@ -543,7 +543,7 @@ int ImageIO::loadJPEGFromMemory (const char* buffer, int bufsize)
     return IMIO_SUCCESS;
 }
 
-int ImageIO::loadJPEG (Glib::ustring fname)
+int ImageIO::loadJPEG (const Glib::ustring &fname)
 {
     FILE *file = g_fopen(fname.c_str (), "rb");
 
@@ -629,7 +629,7 @@ int ImageIO::loadJPEG (Glib::ustring fname)
     }
 }
 
-int ImageIO::getTIFFSampleFormat (Glib::ustring fname, IIOSampleFormat &sFormat, IIOSampleArrangement &sArrangement)
+int ImageIO::getTIFFSampleFormat (const Glib::ustring &fname, IIOSampleFormat &sFormat, IIOSampleArrangement &sArrangement)
 {
 #ifdef WIN32
     wchar_t *wfilename = (wchar_t*)g_utf8_to_utf16 (fname.c_str(), -1, NULL, NULL, NULL);
@@ -704,20 +704,21 @@ int ImageIO::getTIFFSampleFormat (Glib::ustring fname, IIOSampleFormat &sFormat,
                 sFormat = IIOSF_UNSIGNED_SHORT;
                 return IMIO_SUCCESS;
             }
-        } else if (samplesperpixel == 3 && sampleformat == SAMPLEFORMAT_IEEEFP) {
-            /*
-             * Not yet supported
-             *
-             if (bitspersample==16) {
-                sFormat = IIOSF_HALF;
+        } else if ((samplesperpixel == 3 || samplesperpixel == 4) && sampleformat == SAMPLEFORMAT_IEEEFP) {
+            if (bitspersample==16) {
+                sFormat = IIOSF_FLOAT16;
                 return IMIO_SUCCESS;
-            }*/
+            }
+            if (bitspersample == 24) {
+                sFormat = IIOSF_FLOAT24;
+                return IMIO_SUCCESS;
+            }
             if (bitspersample == 32) {
-                sFormat = IIOSF_FLOAT;
+                sFormat = IIOSF_FLOAT32;
                 return IMIO_SUCCESS;
             }
         }
-    } else if (samplesperpixel == 3 && photometric == PHOTOMETRIC_LOGLUV) {
+    } else if ((samplesperpixel == 3 || samplesperpixel == 4) && photometric == PHOTOMETRIC_LOGLUV) {
         if (compression == COMPRESSION_SGILOG24) {
             sFormat = IIOSF_LOGLUV24;
             return IMIO_SUCCESS;
@@ -730,7 +731,7 @@ int ImageIO::getTIFFSampleFormat (Glib::ustring fname, IIOSampleFormat &sFormat,
     return IMIO_VARIANTNOTSUPPORTED;
 }
 
-int ImageIO::loadTIFF (Glib::ustring fname)
+int ImageIO::loadTIFF (const Glib::ustring &fname)
 {
 
     static MyMutex thumbMutex;
@@ -860,7 +861,7 @@ int ImageIO::loadTIFF (Glib::ustring fname)
             }
         }
 
-        setScanline (row, linebuffer, bitspersample, nullptr, nullptr);
+        setScanline (row, linebuffer, bitspersample);
 
         if (pl && !(row % 100)) {
             pl->setProgress ((double)(row + 1) / height);
@@ -971,7 +972,7 @@ void PNGwriteRawProfile(png_struct *ping, png_info *ping_info, const char *profi
 
 } // namespace
 
-int ImageIO::savePNG  (Glib::ustring fname, volatile int bps)
+int ImageIO::savePNG  (const Glib::ustring &fname, int bps) const
 {
     if (getWidth() < 1 || getHeight() < 1) {
         return IMIO_HEADERERROR;
@@ -1026,6 +1027,9 @@ int ImageIO::savePNG  (Glib::ustring fname, volatile int bps)
     if (bps < 0) {
         bps = getBPS ();
     }
+    if (bps > 16) {
+        bps = 16;
+    }
 
     png_set_IHDR(png, info, width, height, bps, PNG_COLOR_TYPE_RGB,
                  PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_BASE);
@@ -1107,7 +1111,7 @@ int ImageIO::savePNG  (Glib::ustring fname, volatile int bps)
 
 
 // Quality 0..100, subsampling: 1=low quality, 2=medium, 3=high
-int ImageIO::saveJPEG (Glib::ustring fname, int quality, int subSamp)
+int ImageIO::saveJPEG (const Glib::ustring &fname, int quality, int subSamp) const
 {
     if (getWidth() < 1 || getHeight() < 1) {
         return IMIO_HEADERERROR;
@@ -1130,7 +1134,7 @@ int ImageIO::saveJPEG (Glib::ustring fname, int quality, int subSamp)
     jerr.pub.error_exit = my_error_exit;
 
     /* Establish the setjmp return context for my_error_exit to use. */
-#if defined( WIN32 ) && defined( __x86_64__ )
+#if defined( WIN32 ) && defined( __x86_64__ ) && !defined(__clang__)
 
     if (__builtin_setjmp(jerr.setjmp_buffer)) {
 #else
@@ -1248,7 +1252,7 @@ int ImageIO::saveJPEG (Glib::ustring fname, int quality, int subSamp)
     unsigned char *row = new unsigned char [rowlen];
 
     /* To avoid memory leaks we establish a new setjmp return context for my_error_exit to use. */
-#if defined( WIN32 ) && defined( __x86_64__ )
+#if defined( WIN32 ) && defined( __x86_64__ ) && !defined(__clang__)
 
     if (__builtin_setjmp(jerr.setjmp_buffer)) {
 #else
@@ -1297,13 +1301,12 @@ int ImageIO::saveJPEG (Glib::ustring fname, int quality, int subSamp)
     return IMIO_SUCCESS;
 }
 
-int ImageIO::saveTIFF (Glib::ustring fname, int bps, bool uncompressed)
+int ImageIO::saveTIFF (const Glib::ustring &fname, int bps, bool isFloat, bool uncompressed) const
 {
     if (getWidth() < 1 || getHeight() < 1) {
         return IMIO_HEADERERROR;
     }
 
-    //TODO: Handling 32 bits floating point output images!
     bool writeOk = true;
     int width = getWidth ();
     int height = getHeight ();
@@ -1455,26 +1458,36 @@ int ImageIO::saveTIFF (Glib::ustring fname, int bps, bool uncompressed)
     TIFFSetField (out, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
     TIFFSetField (out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
     TIFFSetField (out, TIFFTAG_COMPRESSION, uncompressed ? COMPRESSION_NONE : COMPRESSION_ADOBE_DEFLATE);
-    TIFFSetField (out, TIFFTAG_SAMPLEFORMAT, bps == 32 ? SAMPLEFORMAT_IEEEFP : SAMPLEFORMAT_UINT);
+    TIFFSetField (out, TIFFTAG_SAMPLEFORMAT, (bps == 16 || bps == 32) && isFloat ? SAMPLEFORMAT_IEEEFP : SAMPLEFORMAT_UINT);
 
     if (!uncompressed) {
-        TIFFSetField (out, TIFFTAG_PREDICTOR, bps == 32 ? PREDICTOR_FLOATINGPOINT : PREDICTOR_HORIZONTAL);
+        TIFFSetField (out, TIFFTAG_PREDICTOR, (bps == 16 || bps == 32) && isFloat ? PREDICTOR_FLOATINGPOINT : PREDICTOR_HORIZONTAL);
     }
     if (profileData) {
         TIFFSetField (out, TIFFTAG_ICCPROFILE, profileLength, profileData);
     }
 
     for (int row = 0; row < height; row++) {
-        getScanline (row, linebuffer, bps);
+        getScanline (row, linebuffer, bps, isFloat);
 
-        if(needsReverse && !uncompressed && bps == 32) {
-            for(int i = 0; i < lineWidth; i += 4) {
-                char temp = linebuffer[i];
-                linebuffer[i] = linebuffer[i + 3];
-                linebuffer[i + 3] = temp;
-                temp = linebuffer[i + 1];
-                linebuffer[i + 1] = linebuffer[i + 2];
-                linebuffer[i + 2] = temp;
+        if (bps == 16) {
+            if(needsReverse && !uncompressed && isFloat) {
+                for(int i = 0; i < lineWidth; i += 2) {
+                    char temp = linebuffer[i];
+                    linebuffer[i] = linebuffer[i + 1];
+                    linebuffer[i + 1] = temp;
+                }
+            }
+        } else if (bps == 32) {
+            if(needsReverse && !uncompressed) {
+                for(int i = 0; i < lineWidth; i += 4) {
+                    char temp = linebuffer[i];
+                    linebuffer[i] = linebuffer[i + 3];
+                    linebuffer[i + 3] = temp;
+                    temp = linebuffer[i + 1];
+                    linebuffer[i + 1] = linebuffer[i + 2];
+                    linebuffer[i + 2] = temp;
+                }
             }
         }
 
@@ -1582,7 +1595,7 @@ void png_flush(png_structp png_ptr)
     }
 }
 
-int ImageIO::load (Glib::ustring fname)
+int ImageIO::load (const Glib::ustring &fname)
 {
 
     if (hasPngExtension(fname)) {
@@ -1596,7 +1609,7 @@ int ImageIO::load (Glib::ustring fname)
     }
 }
 
-int ImageIO::save (Glib::ustring fname)
+int ImageIO::save (const Glib::ustring &fname) const
 {
     if (hasPngExtension(fname)) {
         return savePNG (fname);
@@ -1608,3 +1621,57 @@ int ImageIO::save (Glib::ustring fname)
         return IMIO_FILETYPENOTSUPPORTED;
     }
 }
+
+void ImageIO::setProgressListener (ProgressListener* l)
+{
+    pl = l;
+}
+
+void ImageIO::setSampleFormat(IIOSampleFormat sFormat)
+{
+    sampleFormat = sFormat;
+}
+
+IIOSampleFormat ImageIO::getSampleFormat() const
+{
+    return sampleFormat;
+}
+
+void ImageIO::setSampleArrangement(IIOSampleArrangement sArrangement)
+{
+    sampleArrangement = sArrangement;
+}
+
+IIOSampleArrangement ImageIO::getSampleArrangement() const
+{
+    return sampleArrangement;
+}
+
+cmsHPROFILE ImageIO::getEmbeddedProfile () const
+{
+    return embProfile;
+}
+
+void ImageIO::getEmbeddedProfileData (int& length, unsigned char*& pdata) const
+{
+    length = loadedProfileLength;
+    pdata = (unsigned char*)loadedProfileData;
+}
+
+MyMutex& ImageIO::mutex ()
+{
+    return imutex;
+}
+
+void ImageIO::deleteLoadedProfileData( )
+{
+    if(loadedProfileData) {
+        if(loadedProfileDataJpg) {
+            free(loadedProfileData);
+        } else {
+            delete[] loadedProfileData;
+        }
+    }
+
+    loadedProfileData = nullptr;
+}
