@@ -68,7 +68,10 @@ inline void fseek (IMFILE* f, int p, int how)
     } else if (how == SEEK_CUR) {
         f->pos += p;
     } else if (how == SEEK_END) {
-        f->pos = f->size + p;
+        if(p <= 0 && -p <= f->size) {
+            f->pos = f->size + p;
+        }
+        return;
     }
 
     if (f->pos < 0  || f->pos > f->size) {
@@ -79,7 +82,7 @@ inline void fseek (IMFILE* f, int p, int how)
 inline int fgetc (IMFILE* f)
 {
 
-    if (f->pos < f->size) {
+    if (LIKELY(f->pos < f->size)) {
         if (f->plistener && ++f->progress_current >= f->progress_next) {
             imfile_update_progress(f);
         }
