@@ -43,14 +43,14 @@ public:
         iImage->increaseRef();
     }
 
-    ~ProcessingJobImpl ()
+    ~ProcessingJobImpl () override
     {
         if (initialImage) {
             initialImage->decreaseRef();
         }
     }
 
-    bool fastPipeline() const { return fast; }
+    bool fastPipeline() const override { return fast; }
 };
 
 }
