@@ -121,10 +121,12 @@ public:
     {
         return image;
     }
-    float** compress_image(int frameNum, bool freeImage = true); // revert to compressed pixels format and release image data
+    float** compress_image(unsigned int frameNum, bool freeImage = true); // revert to compressed pixels format and release image data
     float** data;             // holds pixel values, data[i][j] corresponds to the ith row and jth column
     unsigned prefilters;               // original filters saved ( used for 4 color processing )
     unsigned int getFrameCount() const { return is_raw; }
+
+    double getBaselineExposure() const { return RT_baseline_exposure; }
 protected:
     Glib::ustring filename; // complete filename
     int rotate_deg; // 0,90,180,270 degree of rotation: info taken by dcraw from exif
@@ -167,10 +169,22 @@ public:
     {
         return top_margin;
     }
+
+    int get_rawwidth() const
+    {
+        return raw_width;
+    }
+
     int get_FujiWidth() const
     {
         return fuji_width;
     }
+
+    float const * get_FloatRawImage() const
+    {
+        return float_raw_image;
+    }
+
     eSensorType getSensorType();
 
     void getRgbCam (float rgbcam[3][4]);
@@ -312,6 +326,11 @@ public:
         return filters == 9;
     }
 
+    bool isFloat() const
+    {
+        return float_raw_image;
+    }
+
 public:
     // dcraw functions
     void pre_interpolate()
