@@ -21,23 +21,28 @@
 //#include <giomm.h>
 #include <helpers.h>
 
-class PListener : public rtengine::ProgressListener
+class PListener :
+    public rtengine::ProgressListener
 {
-
 public:
-    void setProgressStr (Glib::ustring str)
+    void setProgressStr(const Glib::ustring& str)
     {
         std::cout << str << std::endl;
     }
-    void setProgress (double p)
+    void setProgress(double p)
     {
         std::cout << p << std::endl;
     }
+    void setProgressState(bool inProcessing)
+    {
+    }
+    void error(const Glib::ustring& descr)
+    {
+    }
 };
 
 int main (int argc, char* argv[])
 {
-
     if (argc < 4) {
         std::cout << "Usage: rtcmd <infile> <paramfile> <outfile>" << std::endl;
         exit(1);
