@@ -33,6 +33,7 @@ extern const Settings *settings;
 LFModifier::~LFModifier()
 {
     if (data_) {
+        MyMutex::MyLock lock(lfModifierMutex);
         data_->Destroy();
     }
 }
@@ -56,7 +57,7 @@ void LFModifier::correctDistortion(double &x, double &y, int cx, int cy, double
     if (swap_xy_) {
         std::swap(xx, yy);
     }
-    if (data_->ApplyGeometryDistortion(xx, yy, 1, 1, pos)) {
+    if (data_->ApplyGeometryDistortion(xx, yy, 1, 1, pos)) {  // This is thread-safe
         x = pos[0];
         y = pos[1];
         if (swap_xy_) {
@@ -95,7 +96,7 @@ void LFModifier::correctCA(double &x, double &y, int cx, int cy, int channel) co
     if (swap_xy_) {
         std::swap(x, y);
     }
-    data_->ApplySubpixelDistortion(x, y, 1, 1, pos);
+    data_->ApplySubpixelDistortion(x, y, 1, 1, pos);  // This is thread-safe
     x = pos[2*channel];
     y = pos[2*channel+1];
     if (swap_xy_) {
@@ -112,12 +113,14 @@ void LFModifier::correctCA(double &x, double &y, int cx, int cy, int channel) co
 
 void LFModifier::processVignetteLine(int width, int y, float *line) const
 {
+    MyMutex::MyLock lock(lfModifierMutex);
     data_->ApplyColorModification(line, 0, y, width, 1, LF_CR_1(INTENSITY), 0);
 }
 
 
 void LFModifier::processVignetteLine3Channels(int width, int y, float *line) const
 {
+    MyMutex::MyLock lock(lfModifierMutex);
     data_->ApplyColorModification(line, 0, y, width, 1, LF_CR_3(RED, GREEN, BLUE), 0);
 }
 
@@ -380,6 +383,7 @@ LFDatabase::LFDatabase():
 LFDatabase::~LFDatabase()
 {
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         data_->Destroy();
     }
 }
@@ -395,6 +399,7 @@ std::vector<LFCamera> LFDatabase::getCameras() const
 {
     std::vector<LFCamera> ret;
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         auto cams = data_->GetCameras();
         while (*cams) {
             ret.emplace_back();
@@ -410,6 +415,7 @@ std::vector<LFLens> LFDatabase::getLenses() const
 {
     std::vector<LFLens> ret;
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         auto lenses = data_->GetLenses();
         while (*lenses) {
             ret.emplace_back();
@@ -425,6 +431,7 @@ LFCamera LFDatabase::findCamera(const Glib::ustring &make, const Glib::ustring &
 {
     LFCamera ret;
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         auto found = data_->FindCamerasExt(make.c_str(), model.c_str());
         if (found) {
             ret.data_ = found[0];
@@ -439,6 +446,7 @@ LFLens LFDatabase::findLens(const LFCamera &camera, const Glib::ustring &name) c
 {
     LFLens ret;
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         auto found = data_->FindLenses(camera.data_, nullptr, name.c_str());
         for (size_t pos = 0; !found && pos < name.size(); ) {
             // try to split the maker from the model of the lens -- we have to
@@ -476,6 +484,7 @@ std::unique_ptr<LFModifier> LFDatabase::getModifier(const LFCamera &camera, cons
 {
     std::unique_ptr<LFModifier> ret;
     if (data_) {
+        MyMutex::MyLock lock(lfDBMutex);
         if (camera && lens) {
             lfModifier *mod = lfModifier::Create(lens.data_, camera.getCropFactor(), width, height);
             int flags = LF_MODIFY_DISTORTION | LF_MODIFY_SCALE | LF_MODIFY_TCA;
@@ -492,7 +501,6 @@ std::unique_ptr<LFModifier> LFDatabase::getModifier(const LFCamera &camera, cons
 
 std::unique_ptr<LFModifier> LFDatabase::findModifier(const LensProfParams &lensProf, const FramesMetaData *idata, int width, int height, const CoarseTransformParams &coarse, int rawRotationDeg)
 {
-    const LFDatabase *db = getInstance();
     Glib::ustring make, model, lens;
     float focallen = idata->getFocalLen();
     if (lensProf.lfAutoMatch()) {
@@ -507,6 +515,11 @@ std::unique_ptr<LFModifier> LFDatabase::findModifier(const LensProfParams &lensP
         model = lensProf.lfCameraModel;
         lens = lensProf.lfLens;
     }
+    if (make.empty() || model.empty() || lens.empty()) {
+        return nullptr;
+    }
+
+    const LFDatabase *db = getInstance();
     LFCamera c = db->findCamera(make, model);
     LFLens l = db->findLens(lensProf.lfAutoMatch() ? c : LFCamera(), lens);
     if (focallen <= 0 && l.data_ && l.data_->MinFocal == l.data_->MaxFocal) {
