@@ -38,7 +38,7 @@ class LFModifier final :
     public NonCopyable
 {
 public:
-    ~LFModifier();
+    ~LFModifier() override;
 
     explicit operator bool() const;
 
@@ -57,6 +57,7 @@ private:
     lfModifier *data_;
     bool swap_xy_;
     int flags_;
+    mutable MyMutex lfModifierMutex;
 };
 
 class LFCamera final
@@ -121,6 +122,7 @@ private:
     LFDatabase();
     bool LoadDirectory(const char *dirname);
 
+    mutable MyMutex lfDBMutex;
     static LFDatabase instance_;
     lfDatabase *data_;
 };
