@@ -36,7 +36,10 @@
 #include "jpeg.h"
 #include "../rtgui/ppversion.h"
 #include "improccoordinator.h"
+#include "settings.h"
 #include <locale.h>
+#include "StopWatch.h"
+#include "median.h"
 
 namespace
 {
@@ -59,11 +62,15 @@ bool checkRawImageThumb (const rtengine::RawImage& raw_image)
 void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4], bool multiThread)
 {
     DCraw::dcrawImage_t image = ri->get_image();
+    const int height = ri->get_iheight();
+    const int width = ri->get_iwidth();
+    const int top_margin = ri->get_topmargin();
+    const int left_margin = ri->get_leftmargin();
+    const int raw_width = ri->get_rawwidth();
+    const bool isFloat = ri->isFloat();
+    const float * const float_raw_image = ri->get_FloatRawImage();
 
     if (ri->isBayer()) {
-        const int height = ri->get_iheight();
-        const int width = ri->get_iwidth();
-
 #ifdef _OPENMP
         #pragma omp parallel for if(multiThread)
 #endif
@@ -73,8 +80,15 @@ void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4],
             int col = 0;
 
             for (; col < width - 1; col += 2) {
-                float val0 = image[row * width + col][c0];
-                float val1 = image[row * width + col + 1][c1];
+                float val0;
+                float val1;
+                if (isFloat) {
+                    val0 = float_raw_image[(row + top_margin) * raw_width + col + left_margin];
+                    val1 = float_raw_image[(row + top_margin) * raw_width + col + left_margin + 1];
+                } else {
+                    val0 = image[row * width + col][c0];
+                    val1 = image[row * width + col + 1][c1];
+                }
                 val0 -= cblack[c0];
                 val1 -= cblack[c1];
                 val0 *= scale_mul[c0];
@@ -84,16 +98,18 @@ void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4],
             }
 
             if (col < width) { // in case width is odd
-                float val0 = image[row * width + col][c0];
+                float val0;
+                if (isFloat) {
+                    val0 = float_raw_image[(row + top_margin) * raw_width + col + left_margin];
+                } else {
+                    val0 = image[row * width + col][c0];
+                }
                 val0 -= cblack[c0];
                 val0 *= scale_mul[c0];
                 image[row * width + col][c0] = rtengine::CLIP (val0);
             }
         }
     } else if (ri->isXtrans()) {
-        const int height = ri->get_iheight();
-        const int width = ri->get_iwidth();
-
 #ifdef _OPENMP
         #pragma omp parallel for if(multiThread)
 #endif
@@ -108,7 +124,12 @@ void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4],
             for (; col < width - 5; col += 6) {
                 for (int i = 0; i < 6; ++i) {
                     const unsigned ccol = c[i];
-                    float val = image[row * width + col + i][ccol];
+                    float val;
+                    if (isFloat) {
+                        val = float_raw_image[(row + top_margin) * raw_width + col + i + left_margin];
+                    } else {
+                        val = image[row * width + col + i][ccol];
+                    }
                     val -= cblack[ccol];
                     val *= scale_mul[ccol];
                     image[row * width + col + i][ccol] = rtengine::CLIP (val);
@@ -117,12 +138,31 @@ void scale_colors (rtengine::RawImage *ri, float scale_mul[4], float cblack[4],
 
             for (; col < width; ++col) { // remaining columns
                 const unsigned ccol = ri->XTRANSFC (row, col);
-                float val = image[row * width + col][ccol];
+                float val;
+                if (isFloat) {
+                    val = float_raw_image[(row + top_margin) * raw_width + col + left_margin];
+                } else {
+                    val = image[row * width + col][ccol];
+                }
                 val -= cblack[ccol];
                 val *= scale_mul[ccol];
                 image[row * width + col][ccol] = rtengine::CLIP (val);
             }
         }
+    } else if (isFloat) {
+#ifdef _OPENMP
+        #pragma omp parallel for if(multiThread)
+#endif
+        for (int row = 0; row < height; ++row) {
+            for (int col = 0; col < width; ++col) {
+                for (int i = 0; i < ri->get_colors(); ++i) {
+                    float val = float_raw_image[(row + top_margin) * raw_width + col + left_margin + i];
+                    val -= cblack[i];
+                    val *= scale_mul[i];
+                    image[row * width + col][i] = val;
+                }
+            }
+        }
     } else {
         const int size = ri->get_iheight() * ri->get_iwidth();
 
@@ -147,6 +187,8 @@ extern Options options;
 namespace rtengine
 {
 
+extern const Settings *settings;
+
 using namespace procparams;
 
 Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, double wbEq, bool inspectorMode)
@@ -243,6 +285,11 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
             printf ("loadFromImage: Unsupported image type \"%s\"!\n", img->getType());
         }
 
+        ProcParams paramsForAutoExp; // Dummy for constructor
+        ImProcFunctions ipf (&paramsForAutoExp, false);
+        ipf.getAutoExp (tpp->aeHistogram, tpp->aeHistCompression, 0.02, tpp->aeExposureCompensation, tpp->aeLightness, tpp->aeContrast, tpp->aeBlack, tpp->aeHighlightCompression, tpp->aeHighlightCompressionThreshold);
+        tpp->aeValid = true;
+
         if (n > 0) {
             ColorTemp cTemp;
 
@@ -261,13 +308,93 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
     return tpp;
 }
 
-Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode)
+
+namespace {
+
+Image8 *load_inspector_mode(const Glib::ustring &fname, RawMetaDataLocation &rml, eSensorType &sensorType, int &w, int &h)
 {
+    BENCHFUN
+    
+    RawImageSource src;
+    int err = src.load(fname, true);
+    if (err) {
+        return nullptr;
+    }
+
+    src.getFullSize(w, h);
+    sensorType = src.getSensorType();
+    
+    ProcParams neutral;
+    neutral.raw.bayersensor.method = RAWParams::BayerSensor::getMethodString(RAWParams::BayerSensor::Method::FAST);
+    neutral.raw.xtranssensor.method = RAWParams::XTransSensor::getMethodString(RAWParams::XTransSensor::Method::FAST);
+    neutral.icm.inputProfile = "(camera)";
+    neutral.icm.workingProfile = options.rtSettings.srgb;
+
+    src.preprocess(neutral.raw, neutral.lensProf, neutral.coarse, false);
+    double thresholdDummy = 0.f;
+    src.demosaic(neutral.raw, false, thresholdDummy);
+
+    PreviewProps pp(0, 0, w, h, 1);
+
+    Imagefloat tmp(w, h);
+    src.getImage(src.getWB(), TR_NONE, &tmp, pp, neutral.toneCurve, neutral.raw);
+    src.convertColorSpace(&tmp, neutral.icm, src.getWB());
+
+    Image8 *img = new Image8(w, h);
+    const float f = 255.f/65535.f;
+#ifdef _OPENMP
+    #pragma omp parallel for
+#endif
+    for (int y = 0; y < h; ++y) {
+        for (int x = 0; x < w; ++x) {
+            float r = tmp.r(y, x);
+            float g = tmp.g(y, x);
+            float b = tmp.b(y, x);
+            // avoid magenta highlights
+            if (r > MAXVALF && b > MAXVALF) {
+                float v = CLIP((r + g + b) / 3.f) * f;
+                img->r(y, x) = img->g(y, x) = img->b(y, x) = v;
+            } else {
+                img->r(y, x) = Color::gamma_srgbclipped(r) * f;
+                img->g(y, x) = Color::gamma_srgbclipped(g) * f;
+                img->b(y, x) = Color::gamma_srgbclipped(b) * f;
+            }
+        }
+    }
+
+    return img;
+}
+
+} // namespace
+
+Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode, bool forHistogramMatching)
+{
+    Thumbnail* tpp = new Thumbnail ();
+    tpp->isRaw = 1;
+    memset (tpp->colorMatrix, 0, sizeof (tpp->colorMatrix));
+    tpp->colorMatrix[0][0] = 1.0;
+    tpp->colorMatrix[1][1] = 1.0;
+    tpp->colorMatrix[2][2] = 1.0;
+
+    if (inspectorMode && !forHistogramMatching && settings->thumbnail_inspector_mode == Settings::ThumbnailInspectorMode::RAW) {
+        Image8 *img = load_inspector_mode(fname, rml, sensorType, w, h);
+        if (!img) {
+            delete tpp;
+            return nullptr;
+        }
+
+        tpp->scale = 1.;
+        tpp->thumbImg = img;
+
+        return tpp;
+    }
+    
     RawImage *ri = new RawImage (fname);
     unsigned int imageNum = 0;
     int r = ri->loadRaw (false, imageNum, false);
 
     if ( r ) {
+        delete tpp;
         delete ri;
         sensorType = ST_NONE;
         return nullptr;
@@ -300,25 +427,36 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 
     // did we succeed?
     if ( err ) {
-        printf ("Could not extract thumb from %s\n", fname.data());
+        if (options.rtSettings.verbose) {
+            std::cout << "Could not extract thumb from " << fname.c_str() << std::endl;
+        }
+        delete tpp;
         delete img;
         delete ri;
         return nullptr;
     }
 
-    Thumbnail* tpp = new Thumbnail ();
-
-    tpp->isRaw = 1;
-    memset (tpp->colorMatrix, 0, sizeof (tpp->colorMatrix));
-    tpp->colorMatrix[0][0] = 1.0;
-    tpp->colorMatrix[1][1] = 1.0;
-    tpp->colorMatrix[2][2] = 1.0;
-
     if (inspectorMode) {
         // Special case, meaning that we want a full sized thumbnail image (e.g. for the Inspector feature)
         w = img->getWidth();
         h = img->getHeight();
         tpp->scale = 1.;
+
+        if (!forHistogramMatching && settings->thumbnail_inspector_mode == Settings::ThumbnailInspectorMode::RAW_IF_NOT_JPEG_FULLSIZE && float(std::max(w, h))/float(std::max(ri->get_width(), ri->get_height())) < 0.9f) {
+            delete img;
+            delete ri;
+            
+            img = load_inspector_mode(fname, rml, sensorType, w, h);
+            if (!img) {
+                delete tpp;
+                return nullptr;
+            }
+
+            tpp->scale = 1.;
+            tpp->thumbImg = img;
+            
+            return tpp;
+        }
     } else {
         if (fixwh == 1) {
             w = h * img->getWidth() / img->getHeight();
@@ -442,6 +580,7 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
     tpp->camwbBlue = tpp->blueMultiplier / pre_mul[2]; //ri->get_pre_mul(2);
     //tpp->defGain = 1.0 / min(ri->get_pre_mul(0), ri->get_pre_mul(1), ri->get_pre_mul(2));
     tpp->defGain = max (scale_mul[0], scale_mul[1], scale_mul[2], scale_mul[3]) / min (scale_mul[0], scale_mul[1], scale_mul[2], scale_mul[3]);
+    tpp->defGain *= std::pow(2, ri->getBaselineExposure());
 
     tpp->gammaCorrected = true;
 
@@ -798,6 +937,11 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
                 }
             }
         }
+        ProcParams paramsForAutoExp; // Dummy for constructor
+        ImProcFunctions ipf (&paramsForAutoExp, false);
+        ipf.getAutoExp (tpp->aeHistogram, tpp->aeHistCompression, 0.02, tpp->aeExposureCompensation, tpp->aeLightness, tpp->aeContrast, tpp->aeBlack, tpp->aeHighlightCompression, tpp->aeHighlightCompressionThreshold);
+        tpp->aeValid = true;
+
         if (ri->get_colors() == 1) {
             pixSum[0] = pixSum[1] = pixSum[2] = 1.;
             n[0] = n[1] = n[2] = 1;
@@ -868,6 +1012,13 @@ Thumbnail::Thumbnail () :
     wbEqual (-1.0),
     wbTempBias (0.0),
     aeHistCompression (3),
+    aeValid(false),
+    aeExposureCompensation(0.0),
+    aeLightness(0),
+    aeContrast(0),
+    aeBlack(0),
+    aeHighlightCompression(0),
+    aeHighlightCompressionThreshold(0),
     embProfileLength (0),
     embProfileData (nullptr),
     embProfile (nullptr),
@@ -994,7 +1145,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     gmi = gm * defGain / mul_lum;
     bmi = bm * defGain / mul_lum;
 
-    // The RAW exposure is not reflected since it's done in preprocessing. If we only have e.g. the chached thumb,
+    // The RAW exposure is not reflected since it's done in preprocessing. If we only have e.g. the cached thumb,
     // that is already preprocessed. So we simulate the effect here roughly my modifying the exposure accordingly
     if (isRaw) {
         rmi *= params.raw.expos;
@@ -1038,12 +1189,17 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
 
         for (int j = 0; j < rwidth; j++) {
             float red = baseImg->r (i, j) * rmi;
-            baseImg->r (i, j) = CLIP (red);
             float green = baseImg->g (i, j) * gmi;
-            baseImg->g (i, j) = CLIP (green);
             float blue = baseImg->b (i, j) * bmi;
-            baseImg->b (i, j) = CLIP (blue);
-
+            
+            // avoid magenta highlights if highlight recovery is enabled
+            if (params.toneCurve.hrenabled && red > MAXVALF && blue > MAXVALF) {
+                baseImg->r(i, j) = baseImg->g(i, j) = baseImg->b(i, j) = CLIP((red + green + blue) / 3.f);
+            } else {
+                baseImg->r(i, j) = CLIP(red);
+                baseImg->g(i, j) = CLIP(green);
+                baseImg->b(i, j) = CLIP(blue);
+            }
         }
     }
 
@@ -1070,9 +1226,8 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
 
     ipf.firstAnalysis (baseImg, params, hist16);
 
-    if (params.fattal.enabled) {
-        ipf.ToneMapFattal02(baseImg);
-    }
+    ipf.dehaze(baseImg);
+    ipf.ToneMapFattal02(baseImg);
     
     // perform transform
     if (ipf.needsTransform()) {
@@ -1086,21 +1241,6 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
         baseImg = trImg;
     }
 
-    // update blurmap
-    SHMap* shmap = nullptr;
-
-    if (params.sh.enabled) {
-        shmap = new SHMap (fw, fh, false);
-        double radius = sqrt (double (fw * fw + fh * fh)) / 2.0;
-        double shradius = params.sh.radius;
-
-        if (!params.sh.hq) {
-            shradius *= radius / 1800.0;
-        }
-
-        shmap->update (baseImg, shradius, ipf.lumimul, params.sh.hq, 16);
-    }
-
     // RGB processing
     double  expcomp = params.toneCurve.expcomp;
     int     bright = params.toneCurve.brightness;
@@ -1109,8 +1249,17 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     int     hlcompr = params.toneCurve.hlcompr;
     int     hlcomprthresh = params.toneCurve.hlcomprthresh;
 
-    if (params.toneCurve.autoexp && aeHistogram) {
-        ipf.getAutoExp (aeHistogram, aeHistCompression, params.toneCurve.clip, expcomp, bright, contr, black, hlcompr, hlcomprthresh);
+    if (params.toneCurve.autoexp) {
+        if (aeValid) {
+            expcomp = aeExposureCompensation;
+            bright = aeLightness;
+            contr = aeContrast;
+            black = aeBlack;
+            hlcompr = aeHighlightCompression;
+            hlcomprthresh = aeHighlightCompressionThreshold;
+        } else if (aeHistogram) {
+            ipf.getAutoExp (aeHistogram, aeHistCompression, 0.02, expcomp, bright, contr, black, hlcompr, hlcomprthresh);
+        }
     }
 
     LUTf curve1 (65536);
@@ -1150,7 +1299,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     bool opautili = false;
 
     if (params.colorToning.enabled) {
-        TMatrix wprof = ICCStore::getInstance()->workingSpaceMatrix (params.icm.working);
+        TMatrix wprof = ICCStore::getInstance()->workingSpaceMatrix (params.icm.workingProfile);
         double wp[3][3] = {
             {wprof[0][0], wprof[0][1], wprof[0][2]},
             {wprof[1][0], wprof[1][1], wprof[1][2]},
@@ -1202,15 +1351,15 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
 
     if (isRaw) {
         cmsHPROFILE dummy;
-        RawImageSource::findInputProfile (params.icm.input, nullptr, camName, &dcpProf, dummy);
+        RawImageSource::findInputProfile (params.icm.inputProfile, nullptr, camName, &dcpProf, dummy);
 
         if (dcpProf) {
-            dcpProf->setStep2ApplyState (params.icm.working, params.icm.toneCurve, params.icm.applyLookTable, params.icm.applyBaselineExposureOffset, as);
+            dcpProf->setStep2ApplyState (params.icm.workingProfile, params.icm.toneCurve, params.icm.applyLookTable, params.icm.applyBaselineExposureOffset, as);
         }
     }
 
     LUTu histToneCurve;
-    ipf.rgbProc (baseImg, labView, nullptr, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, satLimit, satLimitOpacity, ctColorCurve, ctOpacityCurve, opautili, clToningcurve, cl2Toningcurve, customToneCurve1, customToneCurve2, customToneCurvebw1, customToneCurvebw2, rrm, ggm, bbm, autor, autog, autob, expcomp, hlcompr, hlcomprthresh, dcpProf, as, histToneCurve);
+    ipf.rgbProc (baseImg, labView, nullptr, curve1, curve2, curve, params.toneCurve.saturation, rCurve, gCurve, bCurve, satLimit, satLimitOpacity, ctColorCurve, ctOpacityCurve, opautili, clToningcurve, cl2Toningcurve, customToneCurve1, customToneCurve2, customToneCurvebw1, customToneCurvebw2, rrm, ggm, bbm, autor, autog, autob, expcomp, hlcompr, hlcomprthresh, dcpProf, as, histToneCurve);
 
     // freeing up some memory
     customToneCurve1.Reset();
@@ -1220,10 +1369,6 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     customToneCurvebw1.Reset();
     customToneCurvebw2.Reset();
 
-    if (shmap) {
-        delete shmap;
-    }
-
     // luminance histogram update
     if (params.labCurve.contrast != 0) {
         hist16.clear();
@@ -1234,6 +1379,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
             }
     }
 
+    
     // luminance processing
 //  ipf.EPDToneMap(labView,0,6);
 
@@ -1251,11 +1397,14 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
     ipf.chromiLuminanceCurve (nullptr, 1, labView, labView, curve1, curve2, satcurve, lhskcurve, clcurve, lumacurve, utili, autili, butili, ccutili, cclutili, clcutili, dummy, dummy);
 
     ipf.vibrance (labView);
+    ipf.labColorCorrectionRegions(labView);
 
     if ((params.colorappearance.enabled && !params.colorappearance.tonecie) || !params.colorappearance.enabled) {
         ipf.EPDToneMap (labView, 5, 6);
     }
 
+    ipf.softLight(labView);
+
     if (params.colorappearance.enabled) {
         CurveFactory::curveLightBrightColor (
             params.colorappearance.curve,
@@ -1303,7 +1452,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, eSensorT
         ipf.ciecam_02float (cieView, adap, 1, 2, labView, &params, customColCurve1, customColCurve2, customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, sk, execsharp, d, dj, yb, rtt);
         delete cieView;
     }
-
+    
     // color processing
     //ipf.colorCurve (labView, labView);
 
@@ -1792,46 +1941,53 @@ bool Thumbnail::readImage (const Glib::ustring& fname)
 
     Glib::ustring fullFName = fname + ".rtti";
 
-    if (!Glib::file_test (fullFName, Glib::FILE_TEST_EXISTS)) {
+    if (!Glib::file_test(fullFName, Glib::FILE_TEST_EXISTS)) {
         return false;
     }
 
-    FILE* f = g_fopen (fullFName.c_str (), "rb");
+    FILE* f = g_fopen(fullFName.c_str (), "rb");
 
     if (!f) {
         return false;
     }
 
     char imgType[31];  // 30 -> arbitrary size, but should be enough for all image type's name
-    fgets (imgType, 30, f);
-    imgType[strlen (imgType) - 1] = '\0'; // imgType has a \n trailing character, so we overwrite it by the \0 char
+    fgets(imgType, 30, f);
+    imgType[strlen(imgType) - 1] = '\0'; // imgType has a \n trailing character, so we overwrite it by the \0 char
 
     guint32 width, height;
-    fread (&width, 1, sizeof (guint32), f);
-    fread (&height, 1, sizeof (guint32), f);
+
+    if (fread(&width, 1, sizeof(guint32), f) < sizeof(guint32)) {
+        width = 0;
+    }
+
+    if (fread(&height, 1, sizeof(guint32), f) < sizeof(guint32)) {
+        height = 0;
+    }
 
     bool success = false;
 
-    if (!strcmp (imgType, sImage8)) {
-        Image8 *image = new Image8 (width, height);
-        image->readData (f);
-        thumbImg = image;
-        success = true;
-    } else if (!strcmp (imgType, sImage16)) {
-        Image16 *image = new Image16 (width, height);
-        image->readData (f);
-        thumbImg = image;
-        success = true;
-    } else if (!strcmp (imgType, sImagefloat)) {
-        Imagefloat *image = new Imagefloat (width, height);
-        image->readData (f);
-        thumbImg = image;
-        success = true;
-    } else {
-        printf ("readImage: Unsupported image type \"%s\"!\n", imgType);
+    if (std::min(width , height) > 0) {
+        if (!strcmp(imgType, sImage8)) {
+            Image8 *image = new Image8(width, height);
+            image->readData(f);
+            thumbImg = image;
+            success = true;
+        } else if (!strcmp(imgType, sImage16)) {
+            Image16 *image = new Image16(width, height);
+            image->readData(f);
+            thumbImg = image;
+            success = true;
+        } else if (!strcmp(imgType, sImagefloat)) {
+            Imagefloat *image = new Imagefloat(width, height);
+            image->readData(f);
+            thumbImg = image;
+            success = true;
+        } else {
+            printf ("readImage: Unsupported image type \"%s\"!\n", imgType);
+        }
     }
-
-    fclose (f);
+    fclose(f);
     return success;
 }
 
@@ -1878,6 +2034,38 @@ bool Thumbnail::readData  (const Glib::ustring& fname)
                 aeHistCompression   = keyFile.get_integer ("LiveThumbData", "AEHistCompression");
             }
 
+            aeValid = true;
+            if (keyFile.has_key ("LiveThumbData", "AEExposureCompensation")) {
+                aeExposureCompensation = keyFile.get_double ("LiveThumbData", "AEExposureCompensation");
+            } else {
+                aeValid = false;
+            }
+            if (keyFile.has_key ("LiveThumbData", "AELightness")) {
+                aeLightness   = keyFile.get_integer ("LiveThumbData", "AELightness");
+            } else {
+                aeValid = false;
+            }
+            if (keyFile.has_key ("LiveThumbData", "AEContrast")) {
+                aeContrast   = keyFile.get_integer ("LiveThumbData", "AEContrast");
+            } else {
+                aeValid = false;
+            }
+            if (keyFile.has_key ("LiveThumbData", "AEBlack")) {
+                aeBlack   = keyFile.get_integer ("LiveThumbData", "AEBlack");
+            } else {
+                aeValid = false;
+            }
+            if (keyFile.has_key ("LiveThumbData", "AEHighlightCompression")) {
+                aeHighlightCompression   = keyFile.get_integer ("LiveThumbData", "AEHighlightCompression");
+            } else {
+                aeValid = false;
+            }
+            if (keyFile.has_key ("LiveThumbData", "AEHighlightCompressionThreshold")) {
+                aeHighlightCompressionThreshold   = keyFile.get_integer ("LiveThumbData", "AEHighlightCompressionThreshold");
+            } else {
+                aeValid = false;
+            }
+
             if (keyFile.has_key ("LiveThumbData", "RedMultiplier")) {
                 redMultiplier       = keyFile.get_double ("LiveThumbData", "RedMultiplier");
             }
@@ -1951,7 +2139,12 @@ bool Thumbnail::writeData  (const Glib::ustring& fname)
         keyFile.set_double  ("LiveThumbData", "RedAWBMul", redAWBMul);
         keyFile.set_double  ("LiveThumbData", "GreenAWBMul", greenAWBMul);
         keyFile.set_double  ("LiveThumbData", "BlueAWBMul", blueAWBMul);
-        keyFile.set_integer ("LiveThumbData", "AEHistCompression", aeHistCompression);
+        keyFile.set_double  ("LiveThumbData", "AEExposureCompensation", aeExposureCompensation);
+        keyFile.set_integer ("LiveThumbData", "AELightness", aeLightness);
+        keyFile.set_integer ("LiveThumbData", "AEContrast", aeContrast);
+        keyFile.set_integer ("LiveThumbData", "AEBlack", aeBlack);
+        keyFile.set_integer ("LiveThumbData", "AEHighlightCompression", aeHighlightCompression);
+        keyFile.set_integer ("LiveThumbData", "AEHighlightCompressionThreshold", aeHighlightCompressionThreshold);
         keyFile.set_double  ("LiveThumbData", "RedMultiplier", redMultiplier);
         keyFile.set_double  ("LiveThumbData", "GreenMultiplier", greenMultiplier);
         keyFile.set_double  ("LiveThumbData", "BlueMultiplier", blueMultiplier);
@@ -2012,7 +2205,7 @@ bool Thumbnail::readEmbProfile  (const Glib::ustring& fname)
 
                 if (!fseek (f, 0, SEEK_SET)) {
                     embProfileData = new unsigned char[embProfileLength];
-                    fread (embProfileData, 1, embProfileLength, f);
+                    embProfileLength = fread (embProfileData, 1, embProfileLength, f);
                     embProfile = cmsOpenProfileFromMem (embProfileData, embProfileLength);
                 }
             }
@@ -2044,14 +2237,19 @@ bool Thumbnail::writeEmbProfile (const Glib::ustring& fname)
 bool Thumbnail::readAEHistogram  (const Glib::ustring& fname)
 {
 
-    FILE* f = g_fopen (fname.c_str (), "rb");
+    FILE* f = g_fopen(fname.c_str(), "rb");
 
     if (!f) {
-        aeHistogram (0);
+        aeHistogram.reset();
     } else {
-        aeHistogram (65536 >> aeHistCompression);
-        fread (&aeHistogram[0], 1, (65536 >> aeHistCompression)*sizeof (aeHistogram[0]), f);
+        aeHistogram(65536 >> aeHistCompression);
+        const size_t histoBytes = (65536 >> aeHistCompression) * sizeof(aeHistogram[0]);
+        const size_t bytesRead = fread(&aeHistogram[0], 1, histoBytes, f);
         fclose (f);
+        if (bytesRead != histoBytes) {
+            aeHistogram.reset();
+            return false;
+        }
         return true;
     }
 
