@@ -50,6 +50,13 @@ class Thumbnail
     double autoWBTemp, autoWBGreen, wbEqual, wbTempBias;    // autoWBTemp and autoWBGreen are updated each time autoWB is requested and if wbEqual has been modified
     LUTu aeHistogram;
     int  aeHistCompression;
+    bool aeValid;
+    double aeExposureCompensation;
+    int aeLightness;
+    int aeContrast;
+    int aeBlack;
+    int aeHighlightCompression;
+    int aeHighlightCompressionThreshold;
     int embProfileLength;
     unsigned char* embProfileData;
     cmsHPROFILE embProfile;
@@ -76,7 +83,7 @@ public:
     int      getImageWidth  (const procparams::ProcParams& pparams, int rheight, float &ratio);
     void     getDimensions  (int& w, int& h, double& scaleFac);
 
-    static Thumbnail* loadQuickFromRaw (const Glib::ustring& fname, rtengine::RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode = false);
+    static Thumbnail* loadQuickFromRaw (const Glib::ustring& fname, rtengine::RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, bool rotate, bool inspectorMode = false, bool forHistogramMatching = false);
     static Thumbnail* loadFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, eSensorType &sensorType, int &w, int &h, int fixwh, double wbEq, bool rotate, bool forHistogramMatching = false);
     static Thumbnail* loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, double wbEq, bool inspectorMode = false);
     static RawMetaDataLocation loadMetaDataFromRaw (const Glib::ustring& fname);
@@ -100,6 +107,7 @@ public:
     bool readAEHistogram  (const Glib::ustring& fname);
     bool writeAEHistogram (const Glib::ustring& fname);
 
+    bool isAeValid() { return aeValid; };
     unsigned char* getImage8Data();  // accessor to the 8bit image if it is one, which should be the case for the "Inspector" mode.
 
     // Hombre: ... let's hope that proper template can make this cleaner
@@ -148,7 +156,7 @@ public:
         }
 
         return imgPtr;
-    };
+    }
 };
 }
 
