@@ -73,6 +73,7 @@
 #include "sleef.c"
 #include "opthelper.h"
 #include "rt_algo.h"
+#include "rescale.h"
 
 namespace rtengine
 {
@@ -938,72 +939,20 @@ void solve_pde_fft (Array2Df *F, Array2Df *U, Array2Df *buf, bool multithread)/*
  * RT code from here on
  *****************************************************************************/
 
-inline float get_bilinear_value (const Array2Df &src, float x, float y)
+inline void rescale_bilinear (const Array2Df &src, Array2Df &dst, bool multithread)
 {
-    // Get integer and fractional parts of numbers
-    int xi = x;
-    int yi = y;
-    float xf = x - xi;
-    float yf = y - yi;
-    int xi1 = std::min (xi + 1, src.getCols() - 1);
-    int yi1 = std::min (yi + 1, src.getRows() - 1);
-
-    float bl = src (xi, yi);
-    float br = src (xi1, yi);
-    float tl = src (xi, yi1);
-    float tr = src (xi1, yi1);
-
-    // interpolate
-    float b = xf * br + (1.f - xf) * bl;
-    float t = xf * tr + (1.f - xf) * tl;
-    float pxf = yf * t + (1.f - yf) * b;
-    return pxf;
+    rescaleBilinear(src, dst, multithread);
 }
 
-
-void rescale_bilinear (const Array2Df &src, Array2Df &dst, bool multithread)
+inline void rescale_nearest (const Array2Df &src, Array2Df &dst, bool multithread)
 {
-    float col_scale = float (src.getCols()) / float (dst.getCols());
-    float row_scale = float (src.getRows()) / float (dst.getRows());
-
-#ifdef _OPENMP
-    #pragma omp parallel for if (multithread)
-#endif
-
-    for (int y = 0; y < dst.getRows(); ++y) {
-        float ymrs = y * row_scale;
-
-        for (int x = 0; x < dst.getCols(); ++x) {
-            dst (x, y) = get_bilinear_value (src, x * col_scale, ymrs);
-        }
-    }
-}
-
-void rescale_nearest (const Array2Df &src, Array2Df &dst, bool multithread)
-{
-    const int width = src.getCols();
-    const int height = src.getRows();
-    const int nw = dst.getCols();
-    const int nh = dst.getRows();
-
-#ifdef _OPENMP
-    #pragma omp parallel for if (multithread)
-#endif
-
-    for (int y = 0; y < nh; ++y) {
-        int sy = y * height / nh;
-
-        for (int x = 0; x < nw; ++x) {
-            int sx = x * width / nw;
-            dst (x, y) = src (sx, sy);
-        }
-    }
+    rescaleNearest(src, dst, multithread);
 }
 
 
 inline float luminance (float r, float g, float b, TMatrix ws)
 {
-    return r * ws[1][0] + g * ws[1][1] + b * ws[1][2];
+    return Color::rgbLuminance(r, g, b, ws);
 }
 
 
@@ -1065,6 +1014,10 @@ inline int find_fast_dim (int dim)
 
 void ImProcFunctions::ToneMapFattal02 (Imagefloat *rgb)
 {
+    if (!params->fattal.enabled) {
+        return;
+    }
+    
     BENCHFUN
     const int detail_level = 3;
 
@@ -1092,7 +1045,7 @@ void ImProcFunctions::ToneMapFattal02 (Imagefloat *rgb)
     constexpr float luminance_noise_floor = 65.535f;
     constexpr float min_luminance = 1.f;
 
-    TMatrix ws = ICCStore::getInstance()->workingSpaceMatrix (params->icm.working);
+    TMatrix ws = ICCStore::getInstance()->workingSpaceMatrix (params->icm.workingProfile);
 
 #ifdef _OPENMP
     #pragma omp parallel for if(multiThread)
@@ -1104,7 +1057,7 @@ void ImProcFunctions::ToneMapFattal02 (Imagefloat *rgb)
     }
 
     float oldMedian;
-    const float percentile = float(LIM(1, params->fattal.anchor, 100)) / 100.f;
+    const float percentile = float(LIM(params->fattal.anchor, 1, 100)) / 100.f;
     findMinMaxPercentile (Yr.data(), Yr.getRows() * Yr.getCols(), percentile, oldMedian, percentile, oldMedian, multiThread);
     // median filter on the deep shadows, to avoid boosting noise
     // because w2 >= w and h2 >= h, we can use the L buffer as temporary buffer for Median_Denoise()
@@ -1159,15 +1112,15 @@ void ImProcFunctions::ToneMapFattal02 (Imagefloat *rgb)
         for (int x = 0; x < w; x++) {
             int xx = x * wr + 1;
 
-            float Y = std::max(Yr (x, y), epsilon);
-            float l = std::max (L (xx, yy), epsilon) * (scale / Y);
-            rgb->r (y, x) = std::max (rgb->r (y, x), 0.f) * l;
-            rgb->g (y, x) = std::max (rgb->g (y, x), 0.f) * l;
-            rgb->b (y, x) = std::max (rgb->b (y, x), 0.f) * l;
+            float Y = std::max(Yr(x, y), epsilon);
+            float l = std::max(L(xx, yy), epsilon) * (scale / Y);
+            rgb->r(y, x) *= l;
+            rgb->g(y, x) *= l;
+            rgb->b(y, x) *= l;
 
-            assert (std::isfinite (rgb->r (y, x)));
-            assert (std::isfinite (rgb->g (y, x)));
-            assert (std::isfinite (rgb->b (y, x)));
+            assert(std::isfinite(rgb->r(y, x)));
+            assert(std::isfinite(rgb->g(y, x)));
+            assert(std::isfinite(rgb->b(y, x)));
         }
     }
 }
