@@ -32,7 +32,7 @@ namespace rtexif
 class CAOnOffInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         int n = t->toInt();
 
@@ -51,7 +51,7 @@ class CAIntSerNumInterpreter : public Interpreter
 {
 public:
     CAIntSerNumInterpreter () {}
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         return "";
     }
@@ -63,7 +63,7 @@ class CAApertureInterpreter : public Interpreter
 {
 public:
     CAApertureInterpreter () {}
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         double v = pow (2.0, t->toDouble() / 64.0);
@@ -92,7 +92,7 @@ CAMacroModeInterpreter caMacroModeInterpreter;
 class CASelfTimerInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         int sec = t->toInt (0, SHORT);
 
@@ -385,7 +385,7 @@ CAExposureModeInterpreter caExposureModeInterpreter;
 class CAFlashBitsInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         std::ostringstream s;
         unsigned bits = t->toInt (0, SHORT);
@@ -533,7 +533,7 @@ CARAWQualityInterpreter caRAWQualityInterpreter;
 class CAFocalInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         Tag *unitTag = t->getParent()->getRoot()->findTag ("FocalUnits");
         double v = unitTag ? unitTag->toDouble() : 1.;
@@ -567,6 +567,7 @@ public:
             {6, "Sigma 18-125mm f/3.5-5.6 DC IF ASP"},
             {6, "Tokina AF 193-2 19-35mm f/3.5-4.5"},
             {6, "Sigma 28-80mm f/3.5-5.6 II Macro"},
+            {6, "Sigma 28-300mm f/3.5-6.3 DG Macro"},
             {7, "Canon EF 100-300mm f/5.6L"},
             {8, "Canon EF 100-300mm f/5.6 or Sigma or Tokina Lens"},
             {8, "Sigma 70-300mm f/4-5.6 [APO] DG Macro"},
@@ -656,6 +657,9 @@ public:
             {82, "Canon TS-E 135mm f/4L Macro"},
             {94, "Canon TS-E 17mm f/4L"},
             {95, "Canon TS-E 24mm f/3.5L II"},
+            {103, "Samyang AF 14mm f/2.8 EF or Rokinon Lens"},
+            {103, "Rokinon SP 14mm f/2.4"},
+            {103, "Rokinon AF 14mm f/2.8 EF"},
             {124, "Canon MP-E 65mm f/2.8 1-5x Macro Photo"},
             {125, "Canon TS-E 24mm f/3.5L"},
             {126, "Canon TS-E 45mm f/2.8"},
@@ -732,7 +736,7 @@ public:
             {160, "Tokina AT-X 107 AF DX 10-17mm f/3.5-4.5 Fisheye"},
             {160, "Tokina AT-X 116 AF Pro DX 11-16mm f/2.8"},
             {160, "Tokina AT-X 11-20 F2.8 PRO DX Aspherical 11-20mm f/2.8"},
-            {161, "Canon EF 28-70mm f/2.8L USM or Sigma or Tamron Lens"},
+            {161, "Canon EF 28-70mm f/2.8L USM or Other Lens"},
             {161, "Sigma 24-70mm f/2.8 EX"},
             {161, "Sigma 28-70mm f/2.8 EX"},
             {161, "Sigma 24-60mm f/2.8 EX DG"},
@@ -740,6 +744,7 @@ public:
             {161, "Tamron 90mm f/2.8"},
             {161, "Tamron SP AF 17-35mm f/2.8-4 Di LD Aspherical IF"},
             {161, "Tamron SP AF 28-75mm f/2.8 XR Di LD Aspherical [IF] Macro"},
+            {161, "Tokina AT-X 24-70mm f/2.8 PRO FX (IF)"},
             {162, "Canon EF 200mm f/2.8L USM"},
             {163, "Canon EF 300mm f/4L"},
             {164, "Canon EF 400mm f/5.6L"},
@@ -756,6 +761,7 @@ public:
             {169, "Sigma 85mm f/1.4 EX DG HSM"},
             {169, "Sigma 30mm f/1.4 EX DC HSM"},
             {169, "Sigma 35mm f/1.4 DG HSM"},
+            {169, "Sigma 35mm f/1.5 FF High-Speed Prime | 017"},
             {170, "Canon EF 200mm f/2.8L II USM"},
             {171, "Canon EF 300mm f/4L USM"},
             {172, "Canon EF 400mm f/5.6L USM or Sigma Lens"},
@@ -779,6 +785,9 @@ public:
             {180, "Zeiss Milvus 50mm f/1.4"},
             {180, "Zeiss Milvus 85mm f/1.4"},
             {180, "Zeiss Otus 28mm f/1.4 ZE"},
+            {180, "Sigma 24mm f/1.5 FF High-Speed Prime | 017"},
+            {180, "Sigma 50mm f/1.5 FF High-Speed Prime | 017"},
+            {180, "Sigma 85mm f/1.5 FF High-Speed Prime | 017"},
             {181, "Canon EF 100-400mm f/4.5-5.6L IS USM + 1.4x or Sigma Lens"},
             {181, "Sigma 150-600mm f/5-6.3 DG OS HSM | S + 1.4x"},
             {182, "Canon EF 100-400mm f/4.5-5.6L IS USM + 2x or Sigma Lens"},
@@ -790,6 +799,7 @@ public:
             {183, "Sigma 150-600mm f/5-6.3 DG OS HSM | C"},
             {183, "Sigma 150-600mm f/5-6.3 DG OS HSM | S"},
             {183, "Sigma 100-400mm f/5-6.3 DG OS HSM"},
+            {183, "Sigma 180mm f/3.5 APO Macro EX DG IF HSM"},
             {184, "Canon EF 400mm f/2.8L USM + 2x"},
             {185, "Canon EF 600mm f/4L IS USM"},
             {186, "Canon EF 70-200mm f/4L USM"},
@@ -797,7 +807,8 @@ public:
             {188, "Canon EF 70-200mm f/4L USM + 2x"},
             {189, "Canon EF 70-200mm f/4L USM + 2.8x"},
             {190, "Canon EF 100mm f/2.8 Macro USM"},
-            {191, "Canon EF 400mm f/4 DO IS"},
+            {191, "Canon EF 400mm f/4 DO IS or Sigma Lens"},
+            {191, "Sigma 500mm f/4 DG OS HSM"},
             {193, "Canon EF 35-80mm f/4-5.6 USM"},
             {194, "Canon EF 80-200mm f/4.5-5.6 USM"},
             {195, "Canon EF 35-105mm f/4.5-5.6 USM"},
@@ -807,6 +818,7 @@ public:
             {198, "Canon EF 50mm f/1.4 USM or Zeiss Lens"},
             {198, "Zeiss Otus 55mm f/1.4 ZE"},
             {198, "Zeiss Otus 85mm f/1.4 ZE"},
+            {198, "Zeiss Milvus 25mm f/1.4"},
             {199, "Canon EF 28-80mm f/3.5-5.6 USM"},
             {200, "Canon EF 75-300mm f/4-5.6 USM"},
             {201, "Canon EF 28-80mm f/3.5-5.6 USM"},
@@ -840,8 +852,10 @@ public:
             {236, "Canon EF-S 60mm f/2.8 Macro USM"},
             {237, "Canon EF 24-105mm f/4L IS USM"},
             {238, "Canon EF 70-300mm f/4-5.6 IS USM"},
-            {239, "Canon EF 85mm f/1.2L II USM"},
-            {240, "Canon EF-S 17-55mm f/2.8 IS USM"},
+            {239, "Canon EF 85mm f/1.2L II USM or Rokinon Lens"},
+            {239, "Rokinon SP 85mm f/1.2"},
+            {240, "Canon EF-S 17-55mm f/2.8 IS USM or Sigma Lens"},
+            {240, "Sigma 17-50mm f/2.8 EX DC OS HSM"},
             {241, "Canon EF 50mm f/1.2L USM"},
             {242, "Canon EF 70-200mm f/4L IS USM"},
             {243, "Canon EF 70-200mm f/4L IS USM + 1.4x"},
@@ -851,21 +865,33 @@ public:
             {247, "Canon EF 14mm f/2.8L II USM"},
             {248, "Canon EF 200mm f/2L IS USM or Sigma Lens"},
             {248, "Sigma 24-35mm f/2 DG HSM | A"},
+            {248, "Sigma 135mm f/2 FF High-Speed Prime | 017"},
+            {248, "Sigma 24-35mm f/2.2 FF Zoom | 017"},
             {249, "Canon EF 800mm f/5.6L IS USM"},
             {250, "Canon EF 24mm f/1.4L II USM or Sigma Lens"},
             {250, "Sigma 20mm f/1.4 DG HSM | A"},
+            {250, "Sigma 20mm f/1.5 FF High-Speed Prime | 017"},
             {251, "Canon EF 70-200mm f/2.8L IS II USM"},
             {252, "Canon EF 70-200mm f/2.8L IS II USM + 1.4x"},
             {253, "Canon EF 70-200mm f/2.8L IS II USM + 2x"},
             {254, "Canon EF 100mm f/2.8L Macro IS USM"},
             {255, "Sigma 24-105mm f/4 DG OS HSM | A or Other Sigma Lens"},
             {255, "Sigma 180mm f/2.8 EX DG OS HSM APO Macro"},
+            {368, "Sigma 14-24mm f/2.8 DG HSM | A or other Sigma Lens"},
+            {368, "Sigma 20mm f/1.4 DG HSM | A"},
+            {368, "Sigma 50mm f/1.4 DG HSM | A"},
+            {368, "Sigma 40mm f/1.4 DG HSM | A"},
+            {368, "Sigma 60-600mm f/4.5-6.3 DG OS HSM | S"},
             {488, "Canon EF-S 15-85mm f/3.5-5.6 IS USM"},
             {489, "Canon EF 70-300mm f/4-5.6L IS USM"},
             {490, "Canon EF 8-15mm f/4L Fisheye USM"},
             {491, "Canon EF 300mm f/2.8L IS II USM or Tamron Lens"},
             {491, "Tamron SP 70-200mm f/2.8 Di VC USD G2 (A025)"},
             {491, "Tamron 18-400mm f/3.5-6.3 Di II VC HLD (B028)"},
+            {491, "Tamron 100-400mm f/4.5-6.3 Di VC USD (A035)"},
+            {491, "Tamron 70-210mm f/4 Di VC USD (A034)"},
+            {491, "Tamron 70-210mm f/4 Di VC USD (A034) + 1.4x"},
+            {491, "Tamron SP 24-70mm f/2.8 Di VC USD G2 (A032)"},
             {492, "Canon EF 400mm f/2.8L IS II USM"},
             {493, "Canon EF 500mm f/4L IS II USM or EF 24-105mm f4L IS USM"},
             {493, "Canon EF 24-105mm f/4L IS USM"},
@@ -883,12 +909,19 @@ public:
             {508, "Canon EF 11-24mm f/4L USM or Tamron Lens"},
             {508, "Tamron 10-24mm f/3.5-4.5 Di II VC HLD"},
             {747, "Canon EF 100-400mm f/4.5-5.6L IS II USM or Tamron Lens"},
-            {747, "Tamron SP 150-600mm F5-6.3 Di VC USD G2"},
-            {748, "Canon EF 100-400mm f/4.5-5.6L IS II USM + 1.4x"},
+            {747, "Tamron SP 150-600mm f/5-6.3 Di VC USD G2"},
+            {748, "Canon EF 100-400mm f/4.5-5.6L IS II USM + 1.4x or Tamron Lens"},
+            {748, "Tamron 100-400mm f/4.5-6.3 Di VC USD A035E + 1.4x"},
+            {748, "Tamron 70-210mm f/4 Di VC USD (A034) + 2x"},
+            {749, "Tamron 100-400mm f/4.5-6.3 Di VC USD A035E + 2x"},
             {750, "Canon EF 35mm f/1.4L II USM"},
             {751, "Canon EF 16-35mm f/2.8L III USM"},
             {752, "Canon EF 24-105mm f/4L IS II USM"},
             {753, "Canon EF 85mm f/1.4L IS USM"},
+            {754, "Canon EF 70-200mm f/4L IS II USM"},
+            {757, "Canon EF 400mm f/2.8L IS III USM"},
+            {758, "Canon EF 600mm f/4L IS III USM"},
+            {1136, "Sigma 24-70mm f/2.8 DG OS HSM | Art 017"},
             {4142, "Canon EF-S 18-135mm f/3.5-5.6 IS STM"},
             {4143, "Canon EF-M 18-55mm f/3.5-5.6 IS STM or Tamron Lens"},
             {4143, "Tamron 18-200mm f/3.5-6.3 Di III VC"},
@@ -906,9 +939,14 @@ public:
             {4156, "Canon EF 50mm f/1.8 STM"},
             {4157, "Canon EF-M 18-150mm 1:3.5-6.3 IS STM"},
             {4158, "Canon EF-S 18-55mm f/4-5.6 IS STM"},
+            {4159, "Canon EF-M 32mm f/1.4 STM"},
             {4160, "Canon EF-S 35mm f/2.8 Macro IS STM"},
             {36910, "Canon EF 70-300mm f/4-5.6 IS II USM"},
             {36912, "Canon EF-S 18-135mm f/3.5-5.6 IS USM"},
+            {61182, "Canon RF 35mm F1.8 Macro IS STM or other Canon RF Lens"},
+            {61182, "Canon RF 50mm F1.2 L USM"},
+            {61182, "Canon RF 24-105mm F4 L IS USM"},
+            {61182, "Canon RF 28-70mm F2 L USM"},
             {61491, "Canon CN-E 14mm T3.1 L F"},
             {61492, "Canon CN-E 24mm T1.5 L F"},
             {61494, "Canon CN-E 85mm T1.3 L F"},
@@ -918,7 +956,7 @@ public:
         };
     }
 
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         int lensID = t->toInt();
 
@@ -1070,7 +1108,7 @@ CAFocalTypeInterpreter caFocalTypeInterpreter;
 class CAFocalPlaneInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         int val = t->toInt();
 
@@ -1088,7 +1126,7 @@ CAFocalPlaneInterpreter caFocalPlaneInterpreter;
 class CAExposureTimeInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         double d = pow (2, - t->toInt() / 32.0);
@@ -1100,7 +1138,7 @@ CAExposureTimeInterpreter caExposureTimeInterpreter;
 
 class CAEVInterpreter : public Interpreter
 {
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         sprintf (buffer, "%.1f", t->toDouble() / 32.0  );
@@ -1112,14 +1150,14 @@ CAEVInterpreter caEVInterpreter;
 class CABaseISOInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         int a = t->toInt();
         sprintf (buffer, "%d", a);
         return buffer;
     }
-    virtual double toDouble (const Tag* t, int ofs)
+    double toDouble (const Tag* t, int ofs) override
     {
         int a = Interpreter::toInt (t, ofs);
 
@@ -1130,7 +1168,7 @@ public:
             return 0.;
         }
     }
-    virtual int toInt (const Tag* t, int ofs, TagType astype)
+    int toInt (const Tag* t, int ofs, TagType astype) override
     {
         int a = Interpreter::toInt (t, ofs, astype);
 
@@ -1249,7 +1287,7 @@ CASlowShutterInterpreter caSlowShutterInterpreter;
 class CAFlashGuideNumberInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         int n = t->toInt();
 
@@ -1310,7 +1348,7 @@ CAControModeInterpreter caControModeInterpreter;
 class CAFocusDistanceInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         sprintf (buffer, "%.2f", t->toDouble() / 100 );
@@ -1322,7 +1360,7 @@ CAFocusDistanceInterpreter caFocusDistanceInterpreter;
 class CAMeasuredEVInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         char buffer[32];
         sprintf (buffer, "%.1f", t->toDouble() / 8 - 6 );
@@ -1457,7 +1495,7 @@ CAToningEffectInterpreter caToningEffectInterpreter;
 class CAFileNumberInterpreter : public Interpreter
 {
 public:
-    virtual std::string toString (Tag* t)
+    std::string toString (Tag* t) override
     {
         unsigned long val = t->toInt (0, LONG);
         char buffer[32];
@@ -1473,6 +1511,9 @@ class CAModelIDInterpreter : public ChoiceInterpreter
 public:
     CAModelIDInterpreter ()
     {
+        choices[1042] = "EOS M50 / Kiss M";
+        choices[2049] = "PowerShot SX740 HS";
+        choices[2053] = "PowerShot SX70 HS";
         choices[16842752] = "PowerShot A30";
         choices[17039360] = "PowerShot S300 / Digital IXUS 300 / IXY Digital 300";
         choices[17170432] = "PowerShot A20";
@@ -1736,6 +1777,7 @@ public:
         choices[1074257056] = "EOS C300";
         choices[1074257321] = "HF G25";
         choices[1074257844] = "XC10";
+        choices[1074258371] = "EOS C200";
         choices[2147483649] = "EOS-1D";
         choices[2147484007] = "EOS-1DS";
         choices[2147484008] = "EOS 10D";
@@ -1777,7 +1819,7 @@ public:
         choices[2147484452] = "EOS-1D C";
         choices[2147484453] = "EOS 70D";
         choices[2147484454] = "EOS Rebel T5i / 700D / Kiss X7i";
-        choices[2147484455] = "EOS Rebel T5 / 1200D / Kiss X70";
+        choices[2147484455] = "EOS Rebel T5 / 1200D / Kiss X70 / Hi";
         choices[2147484456] = "EOS-1D X MARK II";
         choices[2147484465] = "EOS M";
         choices[2147484486] = "EOS Rebel SL1 / 100D / Kiss X7";
@@ -1793,6 +1835,9 @@ public:
         choices[2147484678] = "EOS 6D Mark II";
         choices[2147484680] = "EOS 77D / 9000D";
         choices[2147484695] = "EOS Rebel SL2 / 200D / Kiss X9";
+        choices[2147484706] = "EOS Rebel T100 / 4000D / 3000D";
+        choices[2147484708] = "EOR R";
+        choices[2147484722] = "EOS Rebel T7 / 2000D / 1500D / Kiss X90";
     }
 };
 CAModelIDInterpreter caModelIDInterpreter;
