@@ -50,8 +50,6 @@ enum {
     ADDSET_SHARPENMICRO_UNIFORMITY,
     ADDSET_VIBRANCE_PASTELS,
     ADDSET_VIBRANCE_SATURATED,
-    ADDSET_FREE_OUPUT_GAMMA,
-    ADDSET_FREE_OUTPUT_SLOPE,
     ADDSET_CAT_DEGREE,
     ADDSET_CAT_ADAPTSCENE,
     ADDSET_CAT_ADAPTVIEWING,
@@ -134,6 +132,17 @@ enum {
     ADDSET_LOCALCONTRAST_DARKNESS,
     ADDSET_LOCALCONTRAST_LIGHTNESS,
     ADDSET_FATTAL_ANCHOR,
+    ADDSET_SHARPENMICRO_CONTRAST,
+    ADDSET_SHARP_CONTRAST,
+	ADDSET_BAYER_FALSE_COLOR_SUPPRESSION,
+	ADDSET_BAYER_ITER,
+	ADDSET_BAYER_PS_SMOOTH,
+	ADDSET_BAYER_PS_EPERISO,
+	ADDSET_BAYER_PS_SIGMA,
+	ADDSET_BAYER_DUALDEMOZCONTRAST,
+	ADDSET_XTRANS_FALSE_COLOR_SUPPRESSION,
+    ADDSET_SOFTLIGHT_STRENGTH,
+    ADDSET_DEHAZE_STRENGTH,
 
     ADDSET_PARAM_NUM // THIS IS USED AS A DELIMITER!!
 };
