@@ -30,21 +30,30 @@ class BayerPreProcess : public ToolParamBlock, public AdjusterListener, public F
 protected:
 
     Adjuster* lineDenoise;
+    MyComboBoxText *lineDenoiseDirection;
     Adjuster* greenEqThreshold;
+    Gtk::CheckButton *pdafLinesFilter;
 
+    rtengine::ProcEvent EvLineDenoiseDirection;
+    rtengine::ProcEvent EvPDAFLinesFilter;
+    
 public:
 
     BayerPreProcess ();
 
-    void read           (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr);
-    void write          (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr);
-    void setBatchMode   (bool batchMode);
-    void setDefaults    (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr);
+    void read           (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write          (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void setBatchMode   (bool batchMode) override;
+    void setDefaults    (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr) override;
+
+    void adjusterChanged(Adjuster* a, double newval) override;
+    void adjusterAutoToggled(Adjuster* a, bool newval) override;
 
-    void adjusterChanged     (Adjuster* a, double newval);
     void hotDeadPixelChanged();
     void setAdjusterBehavior (bool linedenoiseadd, bool greenequiladd);
-    void trimValues          (rtengine::procparams::ProcParams* pp);
+    void trimValues          (rtengine::procparams::ProcParams* pp) override;
+    void lineDenoiseDirectionChanged();
+    void pdafLinesFilterChanged();
 };
 
 #endif
