@@ -25,13 +25,15 @@
 #include "guiutils.h"
 #include "toolpanel.h"
 
-class BayerProcess : public ToolParamBlock, public AdjusterListener, public CheckBoxListener, public FoldableToolPanel, public rtengine::FrameCountListener
+class BayerProcess : public ToolParamBlock, public AdjusterListener, public CheckBoxListener, public FoldableToolPanel, public rtengine::FrameCountListener, public rtengine::AutoContrastListener
 {
 
 protected:
 
     MyComboBoxText* method;
+    Gtk::HBox* borderbox;
     Gtk::HBox *imageNumberBox;
+    Adjuster* border;
     MyComboBoxText* imageNumber;
     Adjuster* ccSteps;
     Gtk::VBox *dcbOptions;
@@ -39,9 +41,10 @@ protected:
     CheckBox* dcbEnhance;
     Gtk::VBox *lmmseOptions;
     Adjuster* lmmseIterations;
-    Gtk::VBox *pixelShiftFrame;
+    Gtk::Frame *pixelShiftFrame;
     Gtk::VBox *pixelShiftOptions;
     MyComboBoxText* pixelShiftMotionMethod;
+    MyComboBoxText* pixelShiftDemosaicMethod;
     CheckBox* pixelShiftShowMotion;
     CheckBox* pixelShiftShowMotionMaskOnly;
     CheckBox* pixelShiftNonGreenCross;
@@ -49,52 +52,42 @@ protected:
     CheckBox* pixelShiftBlur;
     CheckBox* pixelShiftHoleFill;
     CheckBox* pixelShiftMedian;
-//    CheckBox* pixelShiftOneGreen;
-    CheckBox* pixelShiftLmmse;
     CheckBox* pixelShiftEqualBright;
     CheckBox* pixelShiftEqualBrightChannel;
     Adjuster* pixelShiftSmooth;
     Adjuster* pixelShiftEperIso;
     Adjuster* pixelShiftSigma;
-#ifdef PIXELSHIFTDEV
-    Adjuster* pixelShiftSum;
-    Adjuster* pixelShiftMotion;
-    MyComboBoxText* pixelShiftMotionCorrection;
-    CheckBox* pixelShiftAutomatic;
-    CheckBox* pixelShiftNonGreenHorizontal;
-    CheckBox* pixelShiftNonGreenVertical;
-    CheckBox* pixelShiftNonGreenCross2;
-    CheckBox* pixelShiftNonGreenAmaze;
-    CheckBox* pixelShiftExp0;
-    CheckBox* pixelShiftMedian3;
-    Adjuster* pixelShiftStddevFactorGreen;
-    Adjuster* pixelShiftStddevFactorRed;
-    Adjuster* pixelShiftStddevFactorBlue;
-    Adjuster* pixelShiftNreadIso;
-    Adjuster* pixelShiftPrnu;
-    Adjuster* pixelShiftRedBlueWeight;
-#endif
+    Gtk::VBox *dualDemosaicOptions;
+    Adjuster* dualDemosaicContrast;
     int oldMethod;
-
+    bool lastAutoContrast;
     IdleRegister idle_register;
+
+    rtengine::ProcEvent EvDemosaicBorder;
+    rtengine::ProcEvent EvDemosaicAutoContrast;
+    rtengine::ProcEvent EvDemosaicContrast;
+    rtengine::ProcEvent EvDemosaicPixelshiftDemosaicMethod;
 public:
 
     BayerProcess ();
+    ~BayerProcess () override;
 
-    void read           (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr);
-    void write          (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr);
-    void setBatchMode   (bool batchMode);
-    void setDefaults    (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr);
+    void read(const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write(rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void setAdjusterBehavior(bool falsecoloradd, bool iteradd, bool dualdemozecontrastadd, bool pssigmaadd, bool pssmoothadd, bool pseperisoadd);
+    void trimValues(rtengine::procparams::ProcParams* pp) override;
+    void setBatchMode(bool batchMode) override;
+    void setDefaults(const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr) override;
 
-    void methodChanged ();
-    void imageNumberChanged ();
-    void adjusterChanged (Adjuster* a, double newval);
-    void checkBoxToggled (CheckBox* c, CheckValue newval);
+    void methodChanged();
+    void imageNumberChanged();
+    void adjusterChanged(Adjuster* a, double newval) override;
+    void adjusterAutoToggled (Adjuster* a, bool newval) override;
+    void checkBoxToggled(CheckBox* c, CheckValue newval) override;
     void pixelShiftMotionMethodChanged();
-    void FrameCountChanged(int n, int frameNum);
-#ifdef PIXELSHIFTDEV
-    void psMotionCorrectionChanged ();
-#endif
+    void pixelShiftDemosaicMethodChanged();
+    void autoContrastChanged (double autoContrast) override;
+    void FrameCountChanged(int n, int frameNum) override;
 };
 
 #endif
