@@ -39,31 +39,32 @@ class BlackWhite final :
 public:
 
     BlackWhite ();
-    ~BlackWhite ();
+    ~BlackWhite () override;
 
-    void read            (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr);
-    void write           (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr);
-    void setDefaults     (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr);
-    void setBatchMode    (bool batchMode);
-    void autoOpenCurve   ();
-    void setEditProvider (EditDataProvider *provider);
+    void read            (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write           (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void setDefaults     (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr) override;
+    void setBatchMode    (bool batchMode) override;
+    void autoOpenCurve   () override;
+    void setEditProvider (EditDataProvider *provider) override;
 
     void autoch_toggled  ();
     void neutral_pressed ();
 
     void updateRGBLabel      ();
-    void adjusterChanged     (Adjuster* a, double newval);
+    void adjusterChanged     (Adjuster* a, double newval) override;
+    void adjusterAutoToggled (Adjuster* a, bool newval) override;
     void setAdjusterBehavior (bool bwadd, bool bwgadd);
-    void trimValues          (rtengine::procparams::ProcParams* pp);
+    void trimValues          (rtengine::procparams::ProcParams* pp) override;
     void enabledcc_toggled   ();
-    void enabledChanged      ();
+    void enabledChanged      () override;
     void methodChanged       ();
     void filterChanged       ();
     void settingChanged      ();
-    virtual void colorForValue (double valX, double valY, enum ColorCaller::ElemType elemType, int callerId, ColorCaller* caller);
-    void BWChanged           (double redbw, double greenbw, double bluebw);
+    void colorForValue (double valX, double valY, enum ColorCaller::ElemType elemType, int callerId, ColorCaller* caller) override;
+    void BWChanged           (double redbw, double greenbw, double bluebw) override;
     bool BWComputed_         ();
-    void curveChanged        (CurveEditor* ce);
+    void curveChanged        (CurveEditor* ce) override;
     void curveMode1Changed   ();
     bool curveMode1Changed_  ();
     void curveMode1Changed2  ();
@@ -138,6 +139,7 @@ private:
     double nextredbw;
     double nextgreenbw;
     double nextbluebw;
+    int nextcount = 0;
 
     IdleRegister idle_register;
 };
