@@ -36,15 +36,16 @@ public:
 
     ChMixer ();
 
-    void read            (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr);
-    void write           (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr);
-    void setDefaults     (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr);
-    void setBatchMode    (bool batchMode);
+    void read            (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write           (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void setDefaults     (const rtengine::procparams::ProcParams* defParams, const ParamsEdited* pedited = nullptr) override;
+    void setBatchMode    (bool batchMode) override;
 
-    void adjusterChanged (Adjuster* a, double newval);
+    void adjusterChanged (Adjuster* a, double newval) override;
+    void adjusterAutoToggled(Adjuster* a, bool newval) override;
     void setAdjusterBehavior (bool rgbadd);
-    void trimValues          (rtengine::procparams::ProcParams* pp);
-    void enabledChanged();
+    void trimValues          (rtengine::procparams::ProcParams* pp) override;
+    void enabledChanged() override;
 };
 
 #endif
