@@ -7,7 +7,7 @@
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  RawTherapee is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -22,28 +22,29 @@
 #include <gtkmm.h>
 #include "toolpanel.h"
 
-class CoarsePanel : public Gtk::HBox, public ToolPanel {
+class CoarsePanel : public Gtk::HBox, public ToolPanel
+{
 
-    protected:
-        Gtk::Button* rotate_left;
-        Gtk::Button* rotate_right;
-        Gtk::ToggleButton* hflip;
-        Gtk::ToggleButton* vflip;
-        int degree;
-        bool oldhflip, oldvflip, degreechanged;
+protected:
+    Gtk::Button* rotate_left;
+    Gtk::Button* rotate_right;
+    Gtk::ToggleButton* hflip;
+    Gtk::ToggleButton* vflip;
+    int degree;
+    bool oldhflip, oldvflip, degreechanged;
 
-    public:
+public:
 
-        CoarsePanel ();
-    
-        void read               (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited=NULL); 
-        void write              (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited=NULL);
-        void initBatchBehavior  ();
-    
-        void rotateLeft     ();
-        void rotateRight    ();
-        void flipHorizontal ();
-        void flipVertical   ();
+    CoarsePanel ();
+
+    void read               (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write              (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void initBatchBehavior  ();
+
+    void rotateLeft     ();
+    void rotateRight    ();
+    void flipHorizontal ();
+    void flipVertical   ();
 };
 
 #endif
