@@ -57,7 +57,7 @@ public:
     /** @brief Update the position of the edited point ; will trigger events
      *
      * @param pos New position
-     * @param chanIdx Chanel index as given in the std::vector upon instantiation
+     * @param chanIdx Channel index as given in the std::vector upon instantiation
      */
     virtual void setPos(double pos, int chanIdx) = 0;
     virtual void stopNumericalAdjustment() = 0;
@@ -91,7 +91,7 @@ public:
 
         // used to update the AxisAdjuster's parameters
         void updateGUI(const Axis &axis);
-        // useed to update the displayed value
+        // used to update the displayed value
         void setValue(double newValue);
         //bool keyPressed(GdkEventKey* event);
         void valueChanged();
@@ -136,7 +136,7 @@ public:
     /// For more complex adjuster
     CoordinateAdjuster(CoordinateProvider *provider, CurveEditorSubGroup *parent, const std::vector<Axis> &axis);
 
-    virtual ~CoordinateAdjuster();
+    ~CoordinateAdjuster() override;
 
     // Update the Axis list, e.g. on Curve change, but MUST have the same axis count
     void setAxis(const std::vector<Axis> &axis);
