@@ -41,42 +41,43 @@ class Crop final :
 {
 public:
     Crop();
-    ~Crop();
+    ~Crop() override;
 
-    void read           (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr);
-    void write          (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr);
-    void setBatchMode   (bool batchMode);
+    void read           (const rtengine::procparams::ProcParams* pp, const ParamsEdited* pedited = nullptr) override;
+    void write          (rtengine::procparams::ProcParams* pp, ParamsEdited* pedited = nullptr) override;
+    void setBatchMode   (bool batchMode) override;
 
     void ratioChanged   ();
     void ratioFixedChanged ();  // The toggle button
     void refreshSize    ();
     void selectPressed  ();
+    void doresetCrop    ();
     void setDimensions   (int mw, int mh);
-    void enabledChanged ();
+    void enabledChanged () override;
     void positionChanged ();
     void widthChanged   ();
     void heightChanged  ();
     bool refreshSpins   (bool notify = false);
     void notifyListener ();
-    void sizeChanged    (int w, int h, int ow, int oh);
+    void sizeChanged    (int w, int h, int ow, int oh) override;
     void trim           (rtengine::procparams::ProcParams* pp, int ow, int oh);
     void readOptions    ();
     void writeOptions   ();
 
-    void cropMoved          (int &x, int &y, int &w, int &h);
-    void cropWidth1Resized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropWidth2Resized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropHeight1Resized (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropHeight2Resized (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropTopLeftResized     (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropTopRightResized    (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropBottomLeftResized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropBottomRightResized (int &x, int &y, int &w, int &h, float custom_ratio=0.f);
-    void cropInit           (int &x, int &y, int &w, int &h);
-    void cropResized        (int &x, int &y, int& x2, int& y2);
-    void cropManipReady     ();
-    bool inImageArea        (int x, int y);
-    double getRatio         ();
+    void cropMoved          (int &x, int &y, int &w, int &h) override;
+    void cropWidth1Resized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropWidth2Resized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropHeight1Resized (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropHeight2Resized (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropTopLeftResized     (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropTopRightResized    (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropBottomLeftResized  (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropBottomRightResized (int &x, int &y, int &w, int &h, float custom_ratio=0.f) override;
+    void cropInit           (int &x, int &y, int &w, int &h) override;
+    void cropResized        (int &x, int &y, int& x2, int& y2) override;
+    void cropManipReady     () override;
+    bool inImageArea        (int x, int y) override;
+    double getRatio         () const override;
 
     void setCropPanelListener (CropPanelListener* cl)
     {
@@ -94,15 +95,18 @@ private:
         double value;
     };
 
-    const std::vector<CropRatio> crop_ratios;
+    std::vector<CropRatio> crop_ratios;
 
     void adjustCropToRatio();
+    void updateCurrentRatio();
 
     Gtk::CheckButton* fixr;
     MyComboBoxText* ratio;
     MyComboBoxText* orientation;
     MyComboBoxText* guide;
+
     Gtk::Button* selectCrop;
+    Gtk::Button* resetCrop;
     CropPanelListener* clistener;
     int opt;
     MySpinButton* x;
@@ -112,8 +116,10 @@ private:
     MySpinButton* ppi;
     Gtk::Label* sizecm;
     Gtk::Label* sizein;
-    Gtk::VBox* ppibox;
-    Gtk::VBox* sizebox;
+    Gtk::Grid* ppigrid;
+    Gtk::Grid* methodgrid;
+    Gtk::Label *customRatioLabel;
+
     int maxw, maxh;
     double nx, ny;
     int nw, nh;
