@@ -152,7 +152,7 @@ void CurveEditorGroup::newLine()
         if (isHeader) {
             curve_reset = Gtk::manage (new Gtk::Button ());
             setExpandAlignProperties(curve_reset, false, false, Gtk::ALIGN_CENTER, Gtk::ALIGN_FILL);
-            curve_reset->add (*Gtk::manage (new RTImage ("gtk-undo-ltr-small.png", "gtk-undo-rtl-small.png")));
+            curve_reset->add (*Gtk::manage (new RTImage ("undo-small.png", "redo-small.png")));
             curve_reset->set_relief (Gtk::RELIEF_NONE);
             curve_reset->set_tooltip_text (M("CURVEEDITOR_TOOLTIPLINEAR"));
             curve_reset->signal_clicked().connect( sigc::mem_fun(*this, &CurveEditorGroup::curveResetPressed) );
@@ -370,7 +370,7 @@ void CurveEditorGroup::setTooltip( Glib::ustring ttip)
 void CurveEditorGroup::setBatchMode (bool batchMode)
 {
     for (std::vector<CurveEditor*>::iterator i = curveEditors.begin(); i != curveEditors.end(); ++i) {
-        (*i)->curveType->addEntry("unchanged-18.png", M("GENERAL_UNCHANGED"));
+        (*i)->curveType->addEntry("template-24.png", M("GENERAL_UNCHANGED"));
         (*i)->curveType->show();
     }
 }
